/*
 * 新居浜高専PICマイコン学習キットVer.3用モニタプログラム
 * 　　　　作成者：新居浜高専 電子制御工学科 出口幹雄　（2021年11月）
 * 
 * ・XC8 v2.00 でコンパイルしてください。
 * ・Device Family Pack は PIC16F1xxxx_DFP(1.2.99) 
 * ・新居浜高専PICマイコン学習キットVer.3のICSP用ピンソケットの
 * ・GND, PGD, PGC の３ピンを使ってシリアル通信でパソコンとつなぎます。
 *       PGD が RxD （パソコン －－＞ キット）
 *       PGC が TxD （パソコン ＜－－ キット）
 * 　です。
 * ・パソコン側はTeraTermなどのターミナルソフトで、
 *   　9600baud，フロー制御無し，改行コード：CR，文字コード：UTF-8
 * 　に設定してください。
 * ・スイッチを押さえずに電源ONし、デフォルトの時計として動作している状態で、
 * 　SW1，SW2，SW3 の３つを同時押しするとモニタが立ち上がります。
 * ・コマンドでI/Oを操作したり、簡単なプログラムを組んで動作させることができます。
 * ・サンプルプログラム15種類を内蔵しています。
 *
 * [コマンド]
 * "RA="   Aポートにオペランド値(0 ～ 255)を出力する
 * "RAz="(z=3,4,5)   RAzに0または1を出力する
 * "RBz="(z=2,3)   RBzに0または1を出力する（SON時のみ）
 * "RC="   Cポートにオペランド値(0 ～ 255)を出力する
 * "RCz="(z=0～7) 　RCzに0または1を出力する
 * "RAz?"(z=0,1)   RAz(ANAz)の電圧をA/D変換した結果を表示する
 * "RA2?"  RA2から入力した値を0または1で表示する
 * "RB?"  Bポートから読み込んだ値の下位4bitの値(0 ～15)を表示する
 *                                       （RB2,3はSOFF時のみ有効）
 * "RBz?"(z=0～3,5) RBzから入力した値を0または1で表示する
 * "SON"  発音可の状態にする（RB2,3が出力に設定される）
 * "SOFF" 発音禁止の状態にする（RB2,3が入力に設定される）
 * "SDo1" 下のドの音を出す
 * "SRe"  レの音を出す
 * "SMi"  ミの音を出す
 * "SFa"  ファの音を出す
 * "SSo"  ソの音を出す
 * "SRa"  ラの音を出す
 * "SSi"  シの音を出す
 * "SDo2" 上のドの音を出す
 * "wait(z)"(z=1～4998)  およそ0.2msのz倍の時間待つ
 * "print(z)"  zの値を表示する　（例）"print(RA0)"は"RA0?"と同じ動作
 * "send(z)"   zの値を文字コードとして送信する
 * "H"   ヘルプ表示
 * "Hz"(z=1～15)  z番目のサンプルプログラムの説明を表示
 * "L"   プログラムリストを表示する       
 * "G"   プログラムを実行する
 * "E"   プログラムを消去する
 * "Pz"(z=1～15)  z番目のサンプルプログラムをロードする
 * "z:(ステートメント)"　　z行目にステートメントを挿入する
 *                       （z行目以降の行番号はインクリメントされる）
 * "z:"   z行目を削除する（z行目以降は行番号がディクリメントされる）
 * 
 * ［サンプルプログラム］
 * P1: ８つ並んだLEDが２進数を数えるようにカウントアップします。
 *      SW1(RB0)を押すと終了します。
 * P2: 右端の７セグメントLEDが0から9までカウントアップします。
 * P3: ８つ並んだLEDが２進数を数えるように、SW1(RB0)を押すとカウントアップ、
 *      SW2(RB1)を押すとカウントダウン、SW3(RB2)を押すと0にクリア、
 *      SW4(RB3)を押すと終了します。
 * P4: 可変抵抗の電圧(RA0の電圧)をA/D変換して、モニタ上にバーグラフで表示します。
 *     また、８つ並んだLEDの点灯位置が移動します。
 *     SW1(RB0)を押すと終了します。
 * P5: キーボードで数値を入力してEnter、２つめの数値を入力してEnterすると、
 *     入力した２つの数値の和を表示します。
 * P6: キーボードで数値（65535以下）を入力してEnterすると、
 *     その数値の平方根の近似値（整数）を表示します。
 * P7: タッチポイント（TP）を触ると、８つ並んだLEDの点灯が切替わります。
 *      SW1(RB0)を押すと終了します。
 * P8: 電子サイコロです。
 *      SW1(RB0)でスタート／ストップ。
 *      SW2(RB1)を押すと終了します。
 * P9: １～８のキーを押すと、ドレミファソラシドが鳴ります。
 *     ８つ並んだLEDも、押した位置が点灯します。
 *     スペースキーで終了します。
 * P10:チャイムの音が鳴ります。
 * P11:サイレン音が鳴ります。
 *      SW1(RB0)を押すと音程が下がり、離すと上がります。
 *      SW2(RB1)を押すと終了します。
 * P12:CdSを手で覆って暗くすると音が高くなります。
 *      SW1(RB0)を押すと終了します。
 * P13:マイクに大きな音を入れるとフルカラーLEDの色が変わります。
 *      SW1(RB0)を押すと終了します。
 * P14:文字とそのアスキーコードを表示します。
 * P15:押したキーのアスキーコードを、８つ並んだLEDの点灯で表示します。
 *     Enterで終了します。
 *
 *  
 */



#include <pic16f18857.h>

#include <stdlib.h>
#include <stdio.h>


#include "helpstr.h"
#include "sampleprog.h"


#define CMDNUM  37  /* number of commands in cmdarray[][] */
#define PLENMAX 2000    /* maximum program length (characters) */  

/* 時計の１秒の長さを決める定数 */
#define TIMECONST 4999  /* 標準値は 4999   値を大きくすると遅れ、小さくすると進みになる */



void disptitle(void);
void disphelp(void);
void prompt(void);
void xmtmsg(unsigned char m);
unsigned char writeprog(void);
void loadprog(void);
void listprog(void);
unsigned char exeprog(void);
unsigned char generalstat(unsigned char len);
unsigned int calsub(unsigned char eqi, unsigned int v1, unsigned int v2);
unsigned char chkstate(unsigned int chki);
unsigned char chkcmd(unsigned char cp);
unsigned char execmd(unsigned char cn);
unsigned char chkcondition(unsigned char len);
unsigned int evalpara(unsigned char len);
unsigned int evalmember(unsigned char i);
void newline(void);
unsigned char operand(void);
void bin8_to_bcd(unsigned char bin8);


extern void do1(void);
extern void re(void);
extern void mi(void);
extern void fa(void);
extern void so(void);
extern void ra(void);
extern void si(void);
extern void do2(void);

extern unsigned char rcvflg;    /* キー入力フラグ */
extern unsigned char rcvdata;   /* キー入力データ */
extern unsigned char bcd[];     /* bin8_to_bcd()用のBCD値保存場所 */
extern unsigned int t2_icnt;   /* TMR2割り込み回数のカウンタ */
extern unsigned char t1h_0, t1l_0;  /* TMR1の初期値 */
extern unsigned char sound;    /* 音発生用のPORTB出力値 */

unsigned char lnbuf[80];   /* line buffer （１行分のバッファ）*/ 
unsigned char pbuf[PLENMAX];    /* program buffer */  
/*
 * format in pbuf[] (program buffer)
 *  "@1v...statement1....@2v.....statement2...    ...0xFF"
 *      @ : line head mark (0x9F)
 *      v : nest level (number of space for indent)
 *   0xFF : end of program
 */
unsigned int plen;   /* program length */  
unsigned char ci;   /* index of character in line buffer */
unsigned int opr;  /* operand value */
unsigned char sbuf[20]; /* temporary statement buffer */
unsigned char fli[8];   /* character as iteration counter */
unsigned int fln[8];    /* iteration in FOR-loop */
unsigned char nsttyp[8];  /* nest type (1:for-loop, 2:if, 3:else, 4:while-loop)  */   
unsigned char nstl; /* nestiing level of repetition */
unsigned int wi, wj, wk;    /* iteration counter "i", "j", "k" */
unsigned int wm, wn;    /* general purpose integer variable */
unsigned int wp, wq;    /* general purpose integer variable */
unsigned int wa[16];   /* general purpose array variable */
unsigned int sp;    /* sound pitch (higher byte is t1h_0, lower byte is t1l_0) */
unsigned char kb;   /* character in key buffer */
unsigned char perr; /* error code for parameter error */

unsigned char msg1[16]="Invalid Command!";    /* 16 characters */
unsigned char msg2[16]="Invalid Operand!";    /* 16 characters */
unsigned char msg3[13]="Syntax Error!";    /* 13 characters */  
unsigned char msg4[20]="Program Buffer Full!";    /* 20 characters */

unsigned char cmdarray[CMDNUM][5]={
    {3,'R','A','=',  0},    /* 0 */
    {4,'R','A','3','='},    /* 1 */
    {4,'R','A','4','='},    /* 2 */
    {4,'R','A','5','='},    /* 3 */
    {4,'R','B','2','='},    /* 4 */
    {4,'R','B','3','='},    /* 5 */
    {4,'R','B','5','?'},    /* 6 */
    {3,'R','C','=',  0},    /* 7 */
    {4,'R','C','0','='},    /* 8 */
    {4,'R','C','1','='},    /* 9 */
    {4,'R','C','2','='},    /* 10 */
    {4,'R','C','3','='},    /* 11 */
    {4,'R','C','4','='},    /* 12 */
    {4,'R','C','5','='},    /* 13 */
    {4,'R','C','6','='},    /* 14 */
    {4,'R','C','7','='},    /* 15 */
    {4,'R','A','0','?'},    /* 16 */
    {4,'R','A','1','?'},    /* 17 */
    {4,'R','A','2','?'},    /* 18 */
    {3,'R','B','?',  0},    /* 19 */
    {4,'R','B','0','?'},    /* 20 */
    {4,'R','B','1','?'},    /* 21 */
    {4,'R','B','2','?'},    /* 22 */
    {4,'R','B','3','?'},    /* 23 */
    {4,'w','a','i','t'},    /* 24 */
    {3,'S','O','N',  0},    /* 25 */
    {4,'S','O','F','F'},    /* 26 */
    {4,'S','D','o','1'},    /* 27 */
    {3,'S','R','e',  0},    /* 28 */
    {3,'S','M','i',  0},    /* 29 */
    {3,'S','F','a',  0},    /* 30 */
    {3,'S','S','o',  0},    /* 31 */
    {3,'S','R','a',  0},    /* 32 */
    {3,'S','S','i',  0},    /* 33 */
    {4,'S','D','o','2'},    /* 34 */
    {4,'p','r','i','n'},    /* 35 (print)  */
    {4,'s','e','n','d'}     /* 36 */
};

unsigned char statarray[4][6]={
    {3,'f','o','r',  0,  0},    /* 0 */
    {2,'i','f',  0,  0,  0},    /* 1 */
    {4,'e','l','s','e',  0},    /* 2 */    
    {5,'w','h','i','l','e'},    /* 3 */
};



void monitor(void){

    unsigned int i;
    unsigned char cn, errc;
    
    TRISB = 0xAF;
    
    disptitle();    /* show title message */
    prompt();   /* show prompt ">" */
    for(cn=0; cn<80; cn++) lnbuf[cn] = 0;   /* clear line buffer */    
    for(i=0; i<PLENMAX; i++) pbuf[i] = 0xFF;   /* clear program buffer */   
    
    plen = 0;  /* program length */ 
    
    if(rcvflg == 1){cn = rcvdata; rcvflg = 0;}  /* clear key buffer */ 
    
    ci = 0;   /* index of character in line buffer */
    while(1){
        if(rcvflg == 1){    /* when there is a received data in "rcvdata" */
            rcvflg = 0;

            if(rcvdata == 0x08){    /* back space */
                if(ci > 0) ci--;
                lnbuf[ci] = 0;  /* delete data */
                while(!TX1STAbits.TRMT);
                TX1REG = rcvdata;   /* echo back */
                while(!TX1STAbits.TRMT);
                TX1REG = 0x20;   /* delete previous character */
                while(!TX1STAbits.TRMT);
                TX1REG = 0x08;   /* resume cursor position */
            }
            else if(rcvdata == 0x09){   /* TAB */
                for(i=0; i<4; i++){ /* convert to 4-spaces */
                    lnbuf[ci] = 0x20;
                    ci++;
                    while(!TX1STAbits.TRMT);
                    TX1REG = 0x20;   /* echo back 4-spaces */
                } 
            }
            else if(rcvdata >= 0x20){   /* characters other than control code  */
                lnbuf[ci] = rcvdata;    /* store the character */
                ci++;
                while(!TX1STAbits.TRMT);
                TX1REG = rcvdata;   /* echo back */
            }
            if(rcvdata == 0x0d){    /* line end */
                lnbuf[ci] = rcvdata;    /* store "CR" */
                ci++;   /* "ci" is the number of characters in this line  */
                while(!TX1STAbits.TRMT);
                TX1REG = rcvdata;   /* echo back */                
                if((lnbuf[0]>=0x30)&&(lnbuf[0]<=0x39)){ /* when line begin with numeric */
                    if(ci < (PLENMAX - plen)){  /* when enough space is reserved, store the line into pbuf[] */
                        if(writeprog()==0){ /* when error occured */
                            xmtmsg(3);  /* "Syntax Error! */
                        }
                    }
                    else{
                        xmtmsg(4);  /* "Program Buffer Full!" */
                    }
                }
                else if(lnbuf[0]=='H'){ /* display help text */
                    disphelp();
                }
                else if(lnbuf[0]=='P'){ /* load sample program and show list  */
                    loadprog();
                    listprog();
                }
                else if(lnbuf[0]=='L'){ /* display program list */
                    listprog();
                }
                else if(lnbuf[0]=='G'){ /* execute program */
                    errc = exeprog();
                    if(errc > 0) xmtmsg(3);
                }
                else if(lnbuf[0]=='E'){ /* erase program */
                    for(i=0; i<PLENMAX; i++) pbuf[i] = 0xFF;
                    plen = 0;
                }
                else{
                    ci = 0; /* move to head of line */
                    cn = chkcmd(0); /* check command from head of line */
                    if(cn != 0xFF){ /* if command is found, */
                        newline();  /* move to next line,   */
                        errc = execmd(cn); /* and execute that command */
                        if(errc > 0) xmtmsg(2); /* "Invalid Operand" */
                    }
                    else{   /* invalid command */
                        xmtmsg(1);  /* "Invalid Command !" */
                    }
                }
                
                ci = 0; /* reset character counter */
                prompt();
                for(cn=0; cn<80; cn++) lnbuf[cn] = 0;   /* clear line buffer */
                perr = 0;
            }
            
        }
              
    }
    
}

/**********************************************************************************/

 /* 
  * ヘルプテキストの表示 
  */
void disphelp(void){   
    
    unsigned int i;
    unsigned char pnum, u;
    
    pnum = 0;
    if((lnbuf[1]>=0x30)&&(lnbuf[1]<=0x39)){ /* "H"の次の数字を読む */
        pnum = 0x0F & lnbuf[1]; /* H1～H15の場合、pnum:プログラム番号 */
        if((lnbuf[2]>=0x30)&&(lnbuf[2]<=0x39)){
            u = pnum << 3;  /* multiply the first number by 10 */
            pnum <<= 1;
            pnum += u;
            pnum += (0x0F & lnbuf[2]);  /* add the second numer */
        }
        /* H1～H15の場合の処理 */
        if(pnum == 1){
            for(i=0; i<164; i++){   /* 文字数 164 */
                while(!TX1STAbits.TRMT);
                TX1REG = hsprog1[i];
            }
        }
        else if(pnum == 2){
            for(i=0; i<120; i++){   /* 文字数 120 */
                while(!TX1STAbits.TRMT);
                TX1REG = hsprog2[i];
            }
        }
        else if(pnum == 3){
            for(i=0; i<260; i++){   /* 文字数 260 */
                while(!TX1STAbits.TRMT);
                TX1REG = hsprog3[i];
            }
        }
        else if(pnum == 4){
            for(i=0; i<245; i++){   /* 文字数 245 */
                while(!TX1STAbits.TRMT);
                TX1REG = hsprog4[i];
            }
        }
        else if(pnum == 5){
            for(i=0; i<261; i++){   /* 文字数 261 */
                while(!TX1STAbits.TRMT);
                TX1REG = hsprog5[i];
            }
        }
        else if(pnum == 6){
            for(i=0; i<172; i++){   /* 文字数 172 */
                while(!TX1STAbits.TRMT);
                TX1REG = hsprog6[i];
            }
        }
        else if(pnum == 7){
            for(i=0; i<175; i++){   /* 文字数 175 */
                while(!TX1STAbits.TRMT);
                TX1REG = hsprog7[i];
            }
        }
        else if(pnum == 8){
            for(i=0; i<150; i++){   /* 文字数 150 */
                while(!TX1STAbits.TRMT);
                TX1REG = hsprog8[i];
            }
        }
        else if(pnum == 9){
            for(i=0; i<221; i++){   /* 文字数 221 */
                while(!TX1STAbits.TRMT);
                TX1REG = hsprog9[i];
            }
        }
        else if(pnum == 10){
            for(i=0; i<134; i++){   /* 文字数 134 */
                while(!TX1STAbits.TRMT);
                TX1REG = hsprog10[i];
            }
        }
        else if(pnum == 11){
            for(i=0; i<184; i++){   /* 文字数 184 */
                while(!TX1STAbits.TRMT);
                TX1REG = hsprog11[i];
            }
        }
        else if(pnum == 12){
            for(i=0; i<144; i++){   /* 文字数 144 */
                while(!TX1STAbits.TRMT);
                TX1REG = hsprog12[i];
            }
        }
        else if(pnum == 13){
            for(i=0; i<165; i++){   /* 文字数 165 */
                while(!TX1STAbits.TRMT);
                TX1REG = hsprog13[i];
            }
        }
        else if(pnum == 14){
            for(i=0; i<98; i++){   /* 文字数 98 */
                while(!TX1STAbits.TRMT);
                TX1REG = hsprog14[i];
            }
        }
        else if(pnum == 15){
            for(i=0; i<162; i++){   /* 文字数 162 */
                while(!TX1STAbits.TRMT);
                TX1REG = hsprog15[i];
            }
        }
    }
    else{   /* Hコマンドの処理 */
        newline();        
        for(i=0; i<1272; i++){   /* 1ページ目　文字数 1272 */
            while(!TX1STAbits.TRMT);
            TX1REG = help_p1[i];
        }
        while(rcvflg == 0); /* どれかのキーが押されるまで待つ */
        rcvflg = 0;
        newline();
        for(i=0; i<1294; i++){   /* 2ページ目　文字数 1294 */
            while(!TX1STAbits.TRMT);
            TX1REG = help_p2[i];
        }
        while(rcvflg == 0); /* どれかのキーが押されるまで待つ */
        rcvflg = 0;
        newline();
        for(i=0; i<1500; i++){   /* 3ページ目　文字数 1500 */
            while(!TX1STAbits.TRMT);
            TX1REG = help_p3[i];
        }
        while(rcvflg == 0); /* どれかのキーが押されるまで待つ */
        rcvflg = 0;
        newline();
        for(i=0; i<524; i++){   /* 4ページ目　文字数 524 */
            while(!TX1STAbits.TRMT);
            TX1REG = help_p4[i];
        }
        newline();
        newline();
        newline();        
        newline();
    }
    
}

/*============================================================================*/

/*
 * サンプルプログラムをロードする。
 */
void loadprog(void){
    
    unsigned char pnum, u;
    unsigned int p;
    
    pnum = 0;
    if((lnbuf[1]>=0x30)&&(lnbuf[1]<=0x39)){ /* "P"の次の数字を読む */
        pnum = 0x0F & lnbuf[1]; /* pnum:プログラム番号 */
        if((lnbuf[2]>=0x30)&&(lnbuf[2]<=0x39)){
            u = pnum << 3;
            pnum <<= 1;
            pnum += u;
            pnum += (0x0F & lnbuf[2]);
        }
    
        for(p=0; p<PLENMAX; p++) pbuf[p] = 0xFF;   /* clear program buffer */   

        if(pnum==1){    /* load sample program 1 */
            for(p=0; p<85; p++){
                pbuf[p] = prog1[p];
            }
            plen = 85;
        }
        else if(pnum==2){    /* load sample program 2 */
            for(p=0; p<141; p++){
                pbuf[p] = prog2[p];
            }
            plen = 141;
        }
        else if(pnum==3){    /* load sample program 3 */
            for(p=0; p<267; p++){
                pbuf[p] = prog3[p];
            }plen = 267;
        }
        else if(pnum==4){    /* load sample program 4 */
            for(p=0; p<151; p++){
                pbuf[p] = prog4[p];
            }
            plen = 151;
        }     
        else if(pnum==5){    /* load sample program 5 */
            for(p=0; p<443; p++){
                pbuf[p] = prog5[p];
            }
            plen = 443;
        } 
        else if(pnum==6){    /* load sample program 6 */
            for(p=0; p<306; p++){
                pbuf[p] = prog6[p];
            }
            plen = 306;
        } 
        else if(pnum==7){    /* load sample program 7 */
            for(p=0; p<119; p++){
                pbuf[p] = prog7[p];
            }
            plen = 119;
        } 
        else if(pnum==8){    /* load sample program 8 */
            for(p=0; p<244; p++){
                pbuf[p] = prog8[p];
            }
            plen = 244;
        }        
        else if(pnum==9){    /* load sample program 9 */
            for(p=0; p<328; p++){
                pbuf[p] = prog9[p];
            }
            plen = 328;
        }        
        else if(pnum==10){    /* load sample program 10 */
            for(p=0; p<534; p++){
                pbuf[p] = prog10[p];
            }
            plen = 534;
        }           
        else if(pnum==11){    /* load sample program 11 */
            for(p=0; p<213; p++){
                pbuf[p] = prog11[p];
            }
            plen = 213;
        }           
        else if(pnum==12){    /* load sample program 12 */
            for(p=0; p<113; p++){
                pbuf[p] = prog12[p];
            }
            plen = 113;
        }                   
        else if(pnum==13){    /* load sample program 13 */
            for(p=0; p<97; p++){
                pbuf[p] = prog13[p];
            }
            plen = 97;
        }                   
        else if(pnum==14){    /* load sample program 14 */
            for(p=0; p<102; p++){
                pbuf[p] = prog14[p];
            }
            plen = 102;
        }                   
         else if(pnum==15){    /* load sample program 15 */
            for(p=0; p<148; p++){
                pbuf[p] = prog15[p];
            }
            plen = 148;
        }                      
    }
}

/*============================================================================*/


/* 
 * プログラムの１行挿入／削除。
 * lnbuf[]に入力された文字列が数値から始まる場合、
 * 数値を行番号と解釈して、
 * それに続くステートメントをその行に挿入する。
 * ステートメントが無い場合は、その行を削除する。
 * 行を挿入する場合は、それより後の行の行番号は全てインクリメント、
 * 削除する場合は、ディクリメントする。
 * 戻り値は、行番号。
 * 行番号の後に":"が付いていなかったり、バッファフルの場合は 0 を返す。
 */
unsigned char writeprog(void){ /* read line number and add/insert the statement into pbuf[] */
    
    unsigned int  i, w, w2;
    unsigned char f, v, lnum, lnlen, s, nlv;
    
    lnlen = ci; /* number of input characters */
    lnum = 0;   /* line number */
    i = 0;
    f = 1;
    while(f){   /* check line number */
        if((lnbuf[i]>=0x30)&&(lnbuf[i]<=0x39)){
            v = lnum << 3;  /* v = lnum x 8 */
            lnum <<= 1;     /* lnum x2 */
            lnum += v;      /* x10 */
            lnum += (0x0F & lnbuf[i]);
        }
        else if(lnbuf[i] == ':'){
            f = 0;  /* finish loop */
        }
        else{
            f = 0;  /* force break */
            lnum = 0;  /* error occured while reading line number */
        }
        i++;
    }
    if(lnum > 99) lnum = 0; /* limit line number within 1 -- 99 */
    if(lnum > 0){ /* when the line number is read successfully */
        /* search the insert position in pbuf[] */
        s = i;  /* head of statements */
        if(lnbuf[s] < 0x20){  /* if no statement, delete this line */
            w = 0;  /* search the line to be deleted */
            f = 1;
            while(f){
                if(pbuf[w] == 0xFF) f = 0;  /* reach the end of program */
                else if(pbuf[w] == 0x9F){
                    if(lnum == pbuf[w+1]){  /* object line is found */
                        w2 = w + 2; /* search the head of next line */
                        v = 1;
                        while(v){
                            w2++;
                            if(pbuf[w2] == 0x9F) v = 0;
                            if(pbuf[w2] == 0xFF) v = 0;
                        }
                        for(i=0; i<plen-w2; i++){
                            pbuf[w+i] = pbuf[w2+i]; /* move line statements */
                            if(pbuf[w+i-1] == 0x9F) pbuf[w+i]--; /* decrement line number */
                        }
                        pbuf[w+plen-w2] = 0xFF;
                        plen = plen + w - w2; /* renew program length */
                    }
                }
                w++;
            }
            
        }
        else{
            nlv = 0;
            f = 1;
            while(f){   /* check the nest level (number of space at the head of line) */
                if(lnbuf[s] == 0x20){ nlv++; s++;} /* count the number of space */
                else    f = 0;  /* when the 1st character is found */  
            }
            w = 0;
            f = 1;
            while(f){
                if(pbuf[w] == 0xFF){   /* reach the end of program */
                    /* append the line at the end of program  */
                    pbuf[w] = 0x9F;
                    pbuf[w+1] = lnum;   /* line number */     
                    pbuf[w+2] = nlv;    /* nest level (number of space for indent) */
                    for(i=s; i<lnlen; i++){
                        pbuf[w+3+i-s] = lnbuf[i];
                    }        
                    pbuf[w+3+lnlen-s] = 0xFF;
                    plen += lnlen - s + 3;  /* renew program length */
                    f = 0;  /* finish storing line */
                }
                else if(pbuf[w] == 0x9F){    /* when line dilimiter is found */
                    /* next character is its line number */
                    if(lnum <= pbuf[w+1]){    /* when the insert position is found */
                        for(i=plen-1; i>w; i--){
                            pbuf[i+lnlen-s+3] = pbuf[i];    /* open the space to insert the line */
                            if(pbuf[i]==0x9F){
                                if(lnum == pbuf[w+1]) pbuf[i+lnlen-s+4]++; /* increment line number */
                            } 
                        }
                        pbuf[w+lnlen-s+3] = 0x9F;
                        if(lnum == pbuf[w+1]) pbuf[w+lnlen-s+4]++;
                        plen += lnlen - s + 3;  /* renew program length */
                        pbuf[plen] = 0xFF;
                        w++; /* position where the statement is to be inserted */
                        pbuf[w] = lnum;
                        w ++;
                        pbuf[w] = nlv;  /* set nest level */
                        w++;
                        for(i=s; i<lnlen; i++){ /* store the line */
                            pbuf[w+i-s] = lnbuf[i];
                        }
                        f = 0;  /* finish inserting line */
                    }
                }
                w++;  
            }
        }
    }
    
    return lnum;  /* line number */
    
}

/*============================================================================*/

/*
 * pbuf[]内のプログラムのリストを表示する。
 */
void listprog(void){
    
    unsigned int i, pi;
    unsigned char f;
        
    pi = 0;
    f = 1;
    while(f){
        if(pbuf[pi] == 0xFF)  f = 0;    /* reach end of program */
        else if(pbuf[pi] == 0x9F){   /* head of line is found */
            pi++;   /* line number is here */
            bin8_to_bcd(pbuf[pi]);  /* convert the line number to BCD */
            newline();  /* move to next line */
            /* disp line number */
            while(!TX1STAbits.TRMT);
            TX1REG = 0x20; /* space */
            while(!TX1STAbits.TRMT);
            TX1REG = bcd[1] + 0x30; /* digit 10 */
            while(!TX1STAbits.TRMT);
            TX1REG = bcd[0] + 0x30; /* digit 1 */
            while(!TX1STAbits.TRMT);
            TX1REG = ':';
            pi++;   /* nest level is here */
            for(i=0; i<pbuf[pi]; i++){  /* nest level means the number of spaces for indent */
                while(!TX1STAbits.TRMT);
                TX1REG = 0x20;
            }
        }
        else{
            while(!TX1STAbits.TRMT);
            TX1REG = pbuf[pi];  /* show the character  */
        }
        pi++;    /* move to next character */
    }
    
    newline();
    
}

/*============================================================================*/

/*
 * pbuf[]内のプログラムを実行する。
 */
unsigned char exeprog(void){
    
    unsigned int i, pi, rpi[8], w, v;
    unsigned char cnsn[8], rpnst, s, u, x, errc;
    unsigned char f1, f2, f3;
        
/*
 * ===== format in pbuf[] (program buffer)
 *  "@1v...statement1....@2v.....statement2...    ...0xFF"
 *      @ : line head mark (0x9F)
 *      v : nest level (number of space for indent)
 *   0xFF : end of program
 *   The number between @ and v is the line number.
 *   The contents of pbuf[] are copied line by line to lnbuf[], excluding spaces.
 *  
 * ===== Relationship between "nestl" and nest strucure
 *         (nstl=0)
 *      01:while
 *      02:         .
 *      03:         (nstl=1)
 *      04:         for
 *         .        .        .
 *         .        .        (nstl=2)
 *         .        .        if
 *         .        .        .        (nstl=3)
 *         .        .        .        .
 *         .        .        else:
 *         .        .        .        (nstl=3)
 *         .        .        .        .
 *         shallow <--nest level--> deep  
 *          
 *      nstl : index of nest level
 *              nstl=0 --> base line
 *      nsttyp[nstl] : nest type (1:for-loop, 2:if, 3:else, 4:while-loop)        
 *      fli[nstl] : character of iteration index if the nest is for-loop
 *      cnsn[nstl] : number of spaces for indent of currently excuting nest
 *      rpi[nstl] : position to be resumed for repetition  
 * 
 * 
 */
    newline();  /* move to next line */
    pi = 0; /* head position of program */
    cnsn[0] = 0;    /* current nest level */
    nstl = 0;    /* index of nest level  */
    rpnst = 0;   /* when repetition is necessary, rpnst>0 */  

    for(u=0; u<80; u++) lnbuf[u] = 0;   /* clear line buffer */
    
    errc = 0;   /* errc=0, if no error occurred */
    f1 = 1;
    while(f1){  /* continue excuting program while f1=1 */
        if(pbuf[pi] == 0xFF){ /* when reach the end of program */
            if(nstl > 0){   /* if the last line of program is in nest */
                if(nsttyp[nstl] == 1){  /* when this nest is for-loop */
                    if(fli[nstl] == 'i'){wi++; w=wi;}
                    if(fli[nstl] == 'j'){wj++; w=wj;}
                    if(fli[nstl] == 'k'){wk++; w=wk;}
                    if(w < fln[nstl]) pi = rpi[nstl];  /* if repetition is necessary, resume the position to repeat and continue*/
                    else{
                        nstl--; /* finish this for-loop */
                        if(nstl==0) f1 = 0; /* if this nest is the outermost one, quit execution */
                    }
                }
                else if(nsttyp[nstl] == 2){ /* when this nest is if-nest */
                    nstl--;
                }
                else if(nsttyp[nstl] == 3){ /* when this nest is else-nest */
                    nstl--;
                }
                else if(nsttyp[nstl] == 4){  /* when this nest is while-loop */
                    pi = rpi[nstl];  /* resume the position */
                    nstl--;
                }   
            }
            else f1 = 0;
        }
        else if(pbuf[pi] == 0x9F){   /* if head of line is found */
            if(pbuf[pi+2] < cnsn[nstl]){    /* when nest level of this line is shallower than currently executing nest */
               if(nsttyp[nstl] == 1){  /* when executing nest is for-loop */
                    if(fli[nstl] == 'i'){wi++; w=wi;}
                    if(fli[nstl] == 'j'){wj++; w=wj;}
                    if(fli[nstl] == 'k'){wk++; w=wk;}
                    if(w < fln[nstl]){ pi = rpi[nstl];}  /* if repetition is necessary, resume the position to repeat */
                    else{ nstl--; }       /* reach end of repetition */    
                }
                else if(nsttyp[nstl]==2){   /* when this nest is if-case, */
                    nstl--;                 /* escape from this nest */
                }
                else if(nsttyp[nstl]==3){   /* when this nest is else-case, */
                    nstl--;                 /* escape from this nest */
                }
                else if(nsttyp[nstl]==4){    /* when this nest is while-loop */
                    pi = rpi[nstl];  /* resume the position */
                    nstl--;
                }                         
            }
            else{   /* execute this line */
                pi += 3;    /* move to start position of statement */
                s = chkstate(pi); /* check "for", "if", "else", "while" */
                /*===================================================================================*/
                if(s == 0xFF){  /* when not these above */
                    f2 = 1;
                    while(f2){  /* execute the process in this line */
                        u = 0;
                        f3 = 1;
                        while(f3){
                            if(pbuf[pi] != 0x20){ lnbuf[u] = pbuf[pi]; u++;}  /* copy to lnbuf[] with the exception of space */
                            pi++;   /* check the next character */
                            if(pbuf[pi]==';'){f3 = 0; pi++;}   /* reach separator */
                            if(pbuf[pi]==0x9F){f3 = 0; f2 = 0;} /* reach next line */
                            if(pbuf[pi]==0xFF){  /* reach program end */
                                f3 = 0;
                                f2 = 0;
                                if(nstl == 0) f1 = 0;
                            }
                        }
                        lnbuf[u] = 0;   /* put null as end mark of statement */
                        x = u;  /* length of this statement (number of characters except spaces) */
                        ci = 0; /* move to lnbuf[0] */
                        u = chkcmd(0);  /* Is the statement one of commands ? */
                        if(u != 0xFF){
                            errc = execmd(u);    /* when the statement is just one of commands, do it */
                            if(errc > 0){f3=0; f2=0; f1=0;} /* when there is an error, abort executuion */
                        }
                        else{   /* when it is not a command */
                            errc = generalstat(x);  /* it is a general statement and do it */
                            if(errc > 0){f3=0; f2=0; f1=0;} /* when there is an error, abort executuion */
                        }
                        
                    }   /* end of 1-line process */
                    // after finishing while(f2) loop, "pi" points the head of next line 
                    if(nstl > 0){   /* when currently in the nest */
                        /* "pi" points the head of next line */
                        if(pbuf[pi] == 0xFF) rpnst = 1; /* this is the last line of program */
                        else if(pbuf[pi+2] < cnsn[nstl]) rpnst = 2;  /* this is the last line of this nest */ 
                        else rpnst = 0;  /* the next line belongs to the same nest (do nothing) */
                        //
                        if(rpnst > 0){ /* when this is the last line of this nest */
                            if(nsttyp[nstl] == 1){  /* when this nest is for-loop */
                                if(fli[nstl] == 'i'){wi++; w=wi;}
                                if(fli[nstl] == 'j'){wj++; w=wj;}
                                if(fli[nstl] == 'k'){wk++; w=wk;}
                                if(w < fln[nstl]) pi = rpi[nstl];  /* if repetition is necessary, resume the position to repeat */
                                else{ nstl--; }  /* reach end of repetition */ 
                            }
                            else if(nsttyp[nstl]==2){    /* when this nest is if-case */
                                if(chkstate(pi+3) == 2){    /* when the next line is "else"-statement */
                                    f2 = 1;
                                    while(f2){  /* this "else:" is not always in the same nest level */
                                        nstl--; /* step down nest level */
                                        if(pbuf[pi+2]==cnsn[nstl]) f2 = 0;
                                        if(nstl == 0) f2 = 0;
                                    }
                                    f2 = 1;
                                    while(f2){  /* to skip the nest, search the head of line of the same or shallower nest level */
                                        pi++;
                                        if(pbuf[pi] == 0x9F){
                                            if(pbuf[pi+2] <= cnsn[nstl]) f2 = 0;
                                        }
                                        if(pbuf[pi] == 0xFF){ f2 = 0; if(nstl == 0) f1 = 0;}
                                    }
                                }
                                else{
                                    nstl--;
                                    if(pbuf[pi] == 0xFF){ if(nstl == 0) f1 = 0;}
                                }
                            }
                            else if(nsttyp[nstl]==3){    /* when this nest is else-case */
                                nstl--;
                                if(pbuf[pi] == 0xFF){ if(nstl == 0) f1 = 0;}
                            }
                            else if(nsttyp[nstl]==4){    /* when this nest is while-nest */
                                pi = rpi[nstl];  /* resume the position */
                                nstl--;
                            }                        
                        }
                    }

                }/*===================================================================================*/
                else if(s == 0){    /* "for" */
                    nstl++;  /* increase loop nest level */
                    f2 = 1;
                    while(f2){  /* check the character as iteration counter */
                        if(pbuf[pi] == 'i'){fli[nstl]='i'; wi = 0; f2 = 0;}
                        if(pbuf[pi] == 'j'){fli[nstl]='j'; wj = 0; f2 = 0;}
                        if(pbuf[pi] == 'k'){fli[nstl]='k'; wk = 0; f2 = 0;}
                        pi++;
                    }
                    f2 = 1;
                    while(f2){  /* skip " in range" and search "(" */
                        if(pbuf[pi] == '(') f2 = 0;
                        pi++;                        
                    }
                    s = 0;
                    f2 = 1;
                    while(f2){ /* copy the content in (  ) to sbuf[]  */
                        if(pbuf[pi] < 0x20) f2 = 0;
                        else if(pbuf[pi] == ')') f2 = 0;
                        else if(pbuf[pi] == 0x20) pi++;   /* ignore space */
                        else{ sbuf[s] = pbuf[pi]; s++; pi++;}
                    }   /* "s" is the number of characters in (  ) */
                    sbuf[s] = 0;    /* put end mark of the parameter */
                    fln[nstl] = evalpara(s);     /* evaluate the parameter */
                    f2 = 1;
                    while(f2){  /* search the head of 1st line in the nest */
                        pi++;
                        if(pbuf[pi] == 0x9F){
                            rpi[nstl] = pi;   /* remember the position to be resumed for the iteration */
                            f2 = 0;
                        }
                    }
                    cnsn[nstl] = pbuf[pi+2];   /* read nest level of this for-loop nest */
                    nsttyp[nstl] = 1;   /* nest type is for-loop */
                    
                }/*===================================================================================*/
                else if(s == 1){    /* "if" */
                    pi += 2;    /* move to the next character of "if" */
                    /* read the condition description */
                    u = 0;
                    f2 = 1;
                    while(f2){
                        if(pbuf[pi] != 0x20){ lnbuf[u] = pbuf[pi]; u++;}  /* copy to lnbuf[] */
                        if(pbuf[pi] == ':') f2 = 0; /* end of condition description */
                        pi++;
                        if(pbuf[pi] == 0x9F){errc=1; f2=0; f1=0;}   /* when ":" was not found, abort execution */
                    }
                    if(chkcondition(u)){ /* ----- when the condition is satisfied ----- */
                        nstl++; /* increase nest level */
                        f2 = 1;
                        while(f2){  /* search the head of next line */
                            pi++;
                            if(pbuf[pi] == 0x9F) f2 = 0;
                        }                    
                        cnsn[nstl] = pbuf[pi+2]; /* read nest level of the if-case */
                        nsttyp[nstl] = 2;   /* nest type is if-case */
                    }
                    else{   /* ----- when the condition is NOT satisfied ----- */
                        /* search "else"-statement */
                        f2 = 1;
                        while(f2){  /* search the head of line of the same or sallower nest level */
                            pi++;
                            if(pbuf[pi] == 0x9F){
                                if(pbuf[pi+2] <= cnsn[nstl]) f2 = 0;
                            }
                            if(pbuf[pi] == 0xFF){ f2 = 0; if(nstl == 0) f1 = 0;}
                        }
                        if(pbuf[pi] != 0xFF){   /* when statemnet is rest */
                            if(pbuf[pi+2] == cnsn[nstl]){   /* and that statement is in the same nest level */
                                if(chkstate(pi+3) == 2){    /* when that is "else"-satement */
                                    nstl++;
                                    f2 = 1;
                                    while(f2){  /* search the head of next line */
                                        pi++;
                                        if(pbuf[pi] == 0x9F) f2 = 0;
                                    }                    
                                    cnsn[nstl] = pbuf[pi+2]; /* read nest level of the else-case */
                                    nsttyp[nstl] = 3;   /* nest type is else-case */
                                }
                            }
                        }
                    }      
                }/*===================================================================================*/
                else if(s == 2){    /* "else" */
                    /* only skip the nest  */
                    f2 = 1;
                    while(f2){  /* search the head of line of the same or shallower nest level */
                        pi++;
                        if(pbuf[pi] == 0x9F){
                            if(pbuf[pi+2] <= cnsn[nstl]) f2 = 0;
                        }
                        if(pbuf[pi] == 0xFF){ f2 = 0; if(nstl == 0) f1 = 0;}
                    }
                    
                }/*===================================================================================*/
                else if(s == 3){    /* "while" */
                    w = pi - 3; /* memory the position to be resumed (head of this line) */
                    pi += 5;    /* move to the next character of "while" */
                    /* read the condition description */
                    u = 0;
                    f2 = 1;
                    while(f2){
                        if(pbuf[pi] != 0x20){ lnbuf[u] = pbuf[pi]; u++;}    /* copy to lnbuf[] */
                        if(pbuf[pi] == ':') f2 = 0; /* end of condition description */
                        pi++;
                        if(pbuf[pi] == 0x9F){errc=1; f2=0; f1=0;}   /* when ":" was not found, abort execution */
                    }
                    if(chkcondition(u)){ /* when the condition is satisfied */
                        nstl++; /* increase nest level */
                        rpi[nstl] = w;  /* positio to be resumed for the repetition (this "while" statement) */
                        f2 = 1;
                        while(f2){  /* search the head of the 1st line in the nest */
                            pi++;
                            if(pbuf[pi] == 0x9F) f2 = 0;
                        }                    
                        cnsn[nstl] = pbuf[pi+2]; /* read nest level in the this while loop */
                        nsttyp[nstl] = 4;   /* nest type is while-case */

                    }
                    else{   /* when the condition is NOT satisfied */
                        f2 = 1;
                        while(f2){  /* search the head of line of the same or shallower nest level compared with this "while" */
                            pi++;
                            if(pbuf[pi] == 0x9F){
                                if(pbuf[pi+2] <= cnsn[nstl]) f2 = 0;
                            }
                            if(pbuf[pi] == 0xFF){ f2 = 0; if(nstl == 0) f1 = 0;}
                        }
                    }       
                }/*===================================================================================*/
            }
        }
  
    }

    return errc;
    
}

/*============================================================================*/
/*
 * コマンド以外の一般的なステートメント（代入文など）の実行 
 */
unsigned char generalstat(unsigned char len){

    unsigned char i, u, f, eqi, p, errc;
    unsigned int v, w;
    
    errc = 0;
    /* search "=" */
    u = 0;
    f = 0;
    for(i=0; i<len; i++){ if(lnbuf[i]=='='){ eqi = i; f = 1;}}  /* eqi: position of "=" in lnbuf[] */
    
    if(f){  /* when "=" is found */
        if(lnbuf[eqi+1]=='{'){    /* check a[] initial value setting or not */
            u = eqi+2;  /* "={..., ..., ...      ....}" (the 1st value is after "{") */
            i = 0;
            while(f){
                if((lnbuf[u]=='0')&&(lnbuf[u+1]=='x')){ /* value in hex */
                    u++;
                    u++;
                    v = 0;
                    p = 1;
                    while(p){
                        if((lnbuf[u]>=0x30)&&(lnbuf[u]<=0x39)){
                            v <<= 4;  /* 4-bit shift left */
                            v += (0x0F & lnbuf[u]);
                        }
                        else if((lnbuf[u]>=0x41)&&(lnbuf[u]<=0x46)){
                            v <<= 4;  /* 4-bit shift left */
                            v += (lnbuf[u] - 0x37);
                        }
                        else if((lnbuf[u]>=0x61)&&(lnbuf[u]<=0x66)){
                            v <<= 4;  /* 4-bit shift left */
                            v += (lnbuf[u] - 0x57);
                        }
                        else{
                            p = 0;
                            if(lnbuf[u]=='}') f = 0;
                        }
                        u++;
                        if(u > len){p=0; f=0; errc=1;}
                    }
                }
                else{   /* dicimal  */
                    v = 0;
                    p = 1;
                    while(p){
                        if((lnbuf[u]>=0x30)&&(lnbuf[u]<=0x39)){
                            w = v << 3;  /* v = 10 x v */
                            v <<= 1;
                            v += w;
                            v += (0x0F & lnbuf[u]);
                        }
                        else{
                            p = 0;
                            if(lnbuf[u]=='}') f = 0;
                        }
                        u++;
                        if(u > len){p=0; f=0; errc=1;}
                    }   
                }
                wa[i] = v;  /* set value into a[i] */
                i++;
            }/* end of while(f) */
        }
        else{ /* Cases other than the initial value setting of a[] */
            /* calculate the right side */
            for(i=0; i<len-eqi-1; i++) sbuf[i] = lnbuf[eqi+i+1];    /* copy to sbuf[] */
            sbuf[len-eqi-1] = 0;    /* null as end mark of statement */
            v = evalpara(len-eqi-1);  /* "v" is the result of calcutation */
            if(perr == 0){
                /* substitute the result to the left side */
                /* (left side is anything other than commands)  */
                if(lnbuf[0] == 'i'){ 
                    wi = calsub(eqi, wi, v);
                }
                if(lnbuf[0] == 'j'){ 
                    wj = calsub(eqi, wj, v);
                }
                if(lnbuf[0] == 'k'){ 
                    wk = calsub(eqi, wk, v);
                }
                if(lnbuf[0] == 'm'){ 
                    wm = calsub(eqi, wm, v);
                }
                if(lnbuf[0] == 'n'){ 
                    wn = calsub(eqi, wn, v);
                }
                if(lnbuf[0] == 'p'){ 
                    wp = calsub(eqi, wp, v);
                }
                if(lnbuf[0] == 'q'){ 
                    wq = calsub(eqi, wq, v);
                }
                if(lnbuf[0] == 'S'){
                    if(lnbuf[1] == 'P'){
                        sp = calsub(eqi, sp, v);
                        t1h_0 = sp >> 8;
                        t1l_0 = sp & 0x00FF;
                    }
                }
                if(lnbuf[0] == 'a'){    /* valid describtions are a[0] -- a[15] or a[i], a[j], a[k], a[m], a[n], a[p], a[q] */
                    if(lnbuf[1] == '['){
                        if((lnbuf[2]>=0x30)&&(lnbuf[2]<=0x39)){
                            u = 0x0F & lnbuf[2];
                            if(u==1){
                                if((lnbuf[3]>=0x30)&&(lnbuf[3]<=0x35)){
                                    u = 0x0F & lnbuf[3];
                                    u += 10;
                                }
                            }
                            w = wa[u];
                            wa[u] = calsub(eqi, w, v);
                        }
                        else if(lnbuf[2]=='i'){
                            if(wi < 16){
                                w = wa[wi];
                                wa[wi] = calsub(eqi, w, v);
                            }
                        }
                        else if(lnbuf[2]=='j'){
                            if(wj < 16){
                                w = wa[wj];
                                wa[wj] = calsub(eqi, w, v);
                            }
                        }
                        else if(lnbuf[2]=='k'){
                            if(wk < 16){
                                w = wa[wk];
                                wa[wk] = calsub(eqi, w, v);
                            }
                        }
                        else if(lnbuf[2]=='m'){
                            if(wm < 16){
                                w = wa[wm];
                                wa[wm] = calsub(eqi, w, v);
                            }
                        }
                        else if(lnbuf[2]=='n'){
                            if(wn < 16){
                                w = wa[wn];
                                wa[wn] = calsub(eqi, w, v);
                            }
                        }
                        else if(lnbuf[2]=='p'){
                            if(wp < 16){
                                w = wa[wp];
                                wa[wp] = calsub(eqi, w, v);
                            }
                        }
                        else if(lnbuf[2]=='q'){
                            if(wq < 16){
                                w = wa[wq];
                                wa[wq] = calsub(eqi, w, v);
                            }
                        }
                        else{
                            errc = 1;
                        }
                    }
                }
            }
            else{
                errc = perr;
            }
        }
    }
    else{   /* when this statement is not an assignment statement (for example "i++") */
        if(lnbuf[0] == 'i'){
            if(lnbuf[1] == '+'){
                if(lnbuf[2] == '+') wi++;
            }
            else if(lnbuf[1] == '-'){
                if(lnbuf[2] == '-') wi--;
            }
        }
        else if(lnbuf[0] == 'j'){
            if(lnbuf[1] == '+'){
                if(lnbuf[2] == '+') wj++;
            }
            else if(lnbuf[1] == '-'){
                if(lnbuf[2] == '-') wj--;
            }
        }
        else if(lnbuf[0] == 'k'){
            if(lnbuf[1] == '+'){
                if(lnbuf[2] == '+') wk++;
            }
            else if(lnbuf[1] == '-'){
                if(lnbuf[2] == '-') wk--;
            }
        }
        else if(lnbuf[0] == 'm'){
            if(lnbuf[1] == '+'){
                if(lnbuf[2] == '+') wm++;
            }
            else if(lnbuf[1] == '-'){
                if(lnbuf[2] == '-') wm--;
            }
        }
        else if(lnbuf[0] == 'n'){
            if(lnbuf[1] == '+'){
                if(lnbuf[2] == '+') wn++;
            }
            else if(lnbuf[1] == '-'){
                if(lnbuf[2] == '-') wn--;
            }
        }
        else if(lnbuf[0] == 'p'){
            if(lnbuf[1] == '+'){
                if(lnbuf[2] == '+') wp++;
            }
            else if(lnbuf[1] == '-'){
                if(lnbuf[2] == '-') wp--;
            }
        }
        else if(lnbuf[0] == 'q'){
            if(lnbuf[1] == '+'){
                if(lnbuf[2] == '+') wq++;
            }
            else if(lnbuf[1] == '-'){
                if(lnbuf[2] == '-') wq--;
            }
        }
        else if(lnbuf[0]=='a'){  /* general purpose array variable */
            if(lnbuf[1]=='['){
                i = 2;
                if((lnbuf[i]>=0x30)&&(lnbuf[i]<=0x39)){
                    f = 0x0F & lnbuf[i];
                    if(f==1){
                        if((lnbuf[i+1]>=0x30)&&(lnbuf[i+1]<=0x35)){
                            i++;
                            f = 0x0F & lnbuf[i];
                            f += 10;
                        }
                    }
                }
                else if(lnbuf[i]=='i') f = wi;
                else if(lnbuf[i]=='j') f = wj;
                else if(lnbuf[i]=='k') f = wk;
                else if(lnbuf[i]=='m') f = wm;
                else if(lnbuf[i]=='n') f = wn;
                else if(lnbuf[i]=='p') f = wp;
                else if(lnbuf[i]=='q') f = wq;
                i++;
                if(lnbuf[i]==']'){
                    i++;
                    if(lnbuf[i]=='+'){
                        i++;
                        if(lnbuf[i]=='+'){
                            wa[f]++;                             
                        }
                        else{
                            errc = 1;
                        }
                    }
                    else if(lnbuf[i]=='-'){
                        i++;
                        if(lnbuf[i]=='-'){
                            wa[f]--;
                        }
                        else{
                            errc = 1;
                        }
                    }
                    else{
                        errc = 1;
                    }
                }
                else{
                    errc = 1;
                }
            }
            else{
                errc = 1;
            }
        }        
        else{
            errc = 1;
        }
    }
    
    return errc;
    
}

/*============================================================================*/

/*
 * 複号演算子の場合の計算を実行するサブルーチン
 * 単なる代入文の場合は右辺の値を返す
 *    eqi : lnbuf[]内の"="の位置を示す指標値
 *    v1 : 左辺の値 
 *    v2 : 右辺の値
 */
unsigned int calsub(unsigned char eqi, unsigned int v1, unsigned int v2){
    
    /* check the operator and do calculation */
    if(lnbuf[eqi-1] == '+') v1 += v2;
    else if(lnbuf[eqi-1] == '-') v1 -= v2;
    else if(lnbuf[eqi-1] == '&') v1 &= v2;
    else if(lnbuf[eqi-1] == '|') v1 |= v2;
    else if(lnbuf[eqi-1] == '^') v1 ^= v2;
    else if(lnbuf[eqi-1] == '%') v1 %= v2;
    else if(lnbuf[eqi-1] == '*') v1 *= v2;
    else if(lnbuf[eqi-1] == '/') v1 /= v2;
    else if(lnbuf[eqi-1] == '<'){
        if(lnbuf[eqi-2] == '<') v1 <<= v2;
    }
    else if(lnbuf[eqi-1] == '>'){
        if(lnbuf[eqi-2] == '>') v1 >>= v2;
    }
    else v1 = v2;
    
    return v1;
    
}

/*============================================================================*/




/*
 * 条件式の判定
 *    戻り値：0 (falseの場合)，1（trueの場合） 
 */
unsigned char chkcondition(unsigned char len){ /* check condition description */
    
    unsigned char i, f, w, coi, ri;
    unsigned int lpara, rpara;
    
    for(i=0; i<20; i++) sbuf[i] = 0;
    i = 0;
    f = 1;
    while(f){   /* search condition operator */
        i++;
        if(lnbuf[i] == '='){w = 1; ri=i+2; f = 0;}
        if(lnbuf[i] == '!'){w = 2; ri=i+2; f = 0;}
        if(lnbuf[i] == '>'){
            if(lnbuf[i+1] == '='){w = 3; ri=i+2; f = 0;}
            else{w = 4; ri=i+1; f = 0;}
        }
        if(lnbuf[i] == '<'){
            if(lnbuf[i+1] == '='){w = 5; ri=i+2; f = 0;}
            else{w = 6; ri=i+1; f = 0;}
        }
    }/* "==":w=1, "!=":w=2, ">=":W=3, ">":W=4, "<=":w=5, "<":W=6  */
    coi = i;    /* remember the position of condition operator */
    /* evaluate left parameter */
    for(i=0; i<coi; i++) sbuf[i] = lnbuf[i];    /* copy the left parameter to temporary statement buffer */
    sbuf[coi] = 0;
    lpara = evalpara(coi);  /* "coi" is the number of characters of left side parameter */
    
    for(i=0; i<len-ri; i++) sbuf[i] = lnbuf[ri+i];    /* copy the right parameter to temporary statement buffer */
    sbuf[len-ri] = 0;
    rpara = evalpara(len-ri);    /* "len - ri" is the number of characters of right side parameter */
    
    f = 0;
    if(w == 1){
        if(lpara == rpara) f = 1;
    }
    else if(w == 2){
        if(lpara != rpara) f = 1;
    }
    else if(w == 3){
        if(lpara >= rpara) f = 1;
    }
    else if(w == 4){
        if(lpara > rpara) f = 1;
    }
    else if(w == 5){
        if(lpara <= rpara) f = 1;
    }
    else if(w == 6){
        if(lpara < rpara) f = 1;
    }
    
    return f;
    
}

/*============================================================================*/

/*
 * 項または計算式の値を求める
 */
unsigned int evalpara(unsigned char len){
    
    unsigned int para, w;
    unsigned char i, f, otyp, opos;
    
    para = 0;
    /* check whether an operator is contained */
    otyp = 0;   /* default : no operator is contained */
    for(i=0; i<len; i++){
        if(sbuf[i] == '+'){
            if(otyp == '+') otyp = 'i';    /* increment operator */
            else{ otyp = '+';    /* add operator */
                opos = i;
            }
        }
        else if(sbuf[i] == '-'){
            if(otyp == '-') otyp = 'd';    /* decrement operator */
            else{ otyp = '-';    /* subtract operator */
                opos = i;
            }
        }
        else if(sbuf[i] == '&'){ otyp = '&'; opos = i;} /* AND operator */  
        else if(sbuf[i] == '|'){ otyp = '|'; opos = i;} /* OR operator */  
        else if(sbuf[i] == '^'){ otyp = '^'; opos = i;} /* XOR operator */  
        else if(sbuf[i] == '%'){ otyp = '%'; opos = i;} /* modulo operator */  
        else if(sbuf[i] == '*'){ otyp = '*'; opos = i;} /* multiply operator */  
        else if(sbuf[i] == '/'){ otyp = '/'; opos = i;} /* divide operator */  
        else if(sbuf[i] == '<'){
            if(sbuf[i+1] == '<'){ otyp = 'L'; opos = i;}  /* left shift operator */
        }
        else if(sbuf[i] == '>'){
            if(sbuf[i+1] == '>'){ otyp = 'R'; opos = i;}  /* right shift operator */
        }
    }
    
    if(otyp == 0){  /* when no operator is contained */
        para = evalmember(0);
    }
    else if(otyp == '+'){
        para = evalmember(0) + evalmember(opos+1);
    }
    else if(otyp == '-'){
        para = evalmember(0) - evalmember(opos+1);
    }
    else if(otyp == '&'){
        para = evalmember(0) & evalmember(opos+1);
    }
    else if(otyp == '|'){
        para = evalmember(0) | evalmember(opos+1);
    }
    else if(otyp == '^'){
        para = evalmember(0) ^ evalmember(opos+1);
    }
    else if(otyp == '%'){
        para = evalmember(0) % evalmember(opos+1);
    }
    else if(otyp == '*'){
        para = evalmember(0) * evalmember(opos+1);
    }
    else if(otyp == '/'){
        para = evalmember(0) / evalmember(opos+1);
    }
    else if(otyp == 'i'){
        para = evalmember(0) + 1;
    }
    else if(otyp == 'd'){
        para = evalmember(0) - 1;
    }
    else if(otyp == 'L'){
        para = evalmember(0) << evalmember(opos+2);
    }
    else if(otyp == 'R'){
        para = evalmember(0) >> evalmember(opos+2);
    }
    
    
    return para;
    
}

/*============================================================================*/



/*
 * 計算式の各項の値を求める
 */
unsigned int evalmember(unsigned char i){ /* evaluate the value of a menber in statement */
    
    unsigned int value, w;
    unsigned char f;
    
        if(sbuf[i] == 'R'){    /* if "R..." */
            if(sbuf[i+1] == 'A'){  /* if "RA" of "RAx" */
                if(sbuf[i+2] == '0'){  /* "RA0" */
                    ADPCH = 0;        // select ANA0
                    ADCON0bits.ADGO = 1;    // A/D 変換を開始する //
                    while(ADCON0bits.ADGO);
                    value = (ADRESH << 8);
                    value += ADRESL;
                }
                else if(sbuf[i+2] == '1'){  /* "RA1" */
                    ADPCH = 1;        // select ANA1
                    ADCON0bits.ADGO = 1;    // A/D 変換を開始する //
                    while(ADCON0bits.ADGO);
                    value = (ADRESH << 8);
                    value += ADRESL;
                }
                else if(sbuf[i+2] == '2'){ /* "RA2" */
                    value = PORTAbits.RA2;
                }
                else{   /* "RA" */
                    value = PORTA & 0x04;    /* extract RA2 */
                }   
            }
            else if(sbuf[i+1] == 'B'){ /* if "RB" or "RBx" */
                if(sbuf[i+2] == '0'){ /* "RB0" */
                    value = PORTBbits.RB0;
                }
                else if(sbuf[i+2] == '1'){ /* "RB1" */
                    value = PORTBbits.RB1;
                }
                else if(sbuf[i+2] == '2'){ /* "RB2" */
                    value = PORTBbits.RB2;
                }
                else if(sbuf[i+2] == '3'){ /* "RB3" */
                    value = PORTBbits.RB3;
                }
                else if(sbuf[i+2] == '5'){ /* "RB5" */
                    value = PORTBbits.RB5;
                }
                else{   /* "RB" */
                    value = PORTB & 0x0F;    /* extract RB0 -- RB3 */
                }         
            }
            else{
                perr = 2;   /* invalid port selection */
            }
        }
        else if(sbuf[i] == 'i') value = wi;
        else if(sbuf[i] == 'j') value = wj;
        else if(sbuf[i] == 'k') value = wk;
        else if(sbuf[i] == 'm') value = wm;
        else if(sbuf[i] == 'n') value = wn;
        else if(sbuf[i] == 'p') value = wp;
        else if(sbuf[i] == 'q') value = wq;
        else if(sbuf[i] == '0'){
            i++;
            if(sbuf[i] == 'x'){   /* hexadicimal numeric */
                i++;
                value = 0;
                f = 1;
                while(f){
                    if((sbuf[i]>=0x30)&&(sbuf[i]<=0x39)){
                        value <<= 4;  /* 4-bit shift left */
                        value += (0x0F & sbuf[i]);
                    }
                    else if((sbuf[i]>=0x41)&&(sbuf[i]<=0x46)){
                        value <<= 4;  /* 4-bit shift left */
                        value += (sbuf[i] - 0x37);
                    }
                    else if((sbuf[i]>=0x61)&&(sbuf[i]<=0x66)){
                        value <<= 4;  /* 4-bit shift left */
                        value += (sbuf[i] - 0x57);
                    }
                    else f = 0;
                    i++;
                }  
            }
            else{   /* dicimal which begins with "0" */
                value = 0;
                f = 1;
                while(f){
                    if((sbuf[i]>=0x30)&&(sbuf[i]<=0x39)){
                        w = value << 3;  /* value = 10 x value */
                        value <<= 1;
                        value += w;
                        value += (0x0F & sbuf[i]);
                    }
                    else f = 0;
                    i++;
                }              
            }
        }
        else if((sbuf[i]>=0x31)&&(sbuf[i]<=0x39)){    /* dicimal numeric */
            value = 0;
            f = 1;
            while(f){
                if((sbuf[i]>=0x30)&&(sbuf[i]<=0x39)){
                    w = value << 3;  /* value = 10 x value */
                    value <<= 1;
                    value += w;
                    value += (0x0F & sbuf[i]);
                }
                else f = 0;
                i++;
            }                 
        }
        else if(sbuf[i]=='K'){  /* a character in the key buffer */
            if(sbuf[i+1]=='B'){
                if(rcvflg ==1){
                    rcvflg = 0;
                    value = rcvdata;
                }
                else value = 0; /* when no key input, return 0 */
            }
        }
        else if(sbuf[i]=='S'){  /* sound pitch parameter */
            if(sbuf[i+1]=='P'){
                value = t1h_0 << 8;
                value += t1l_0;
            }
        }
        else if(sbuf[i]=='a'){  /* general purpose array variable */
            i++;
            if(sbuf[i]=='['){
                i++;
                if((sbuf[i]>=0x30)&&(sbuf[i]<=0x39)){
                    f = 0x0F & sbuf[i];
                    if(f==1){
                        if((sbuf[i+1]>=0x30)&&(sbuf[i+1]<=0x35)){
                            f = 0x0F & sbuf[i+1];
                            f += 10;
                        }
                    }
                    value = wa[f];                    
                }
                else if(sbuf[i]=='i') value = wa[wi];
                else if(sbuf[i]=='j') value = wa[wj];
                else if(sbuf[i]=='k') value = wa[wk];
                else if(sbuf[i]=='m') value = wa[wm];
                else if(sbuf[i]=='n') value = wa[wn];
                else if(sbuf[i]=='p') value = wa[wp];
                else if(sbuf[i]=='q') value = wa[wq];
                
            }
        }
        else{
            perr = 1;   /* invalid member description is found */
            value = 0;
        }
    
    return value;
    
}

/*============================================================================*/



/*
 * ステートメントが "for", "if", "else", "while" のいずれかどうかを調べる
 *     戻り値：0 (for) , 1 (if),  2 (else), 3 (while), 0xFF (いずれでもない)
 */
unsigned char chkstate(unsigned int chki){ /* check statement in pbuf[] */

    unsigned int i;
    unsigned char j, k, m, result;
    
    result = 0xFF;
    for(j=0; j<4; j++){
        i = chki;
        m = 0;
        for(k=0; k<statarray[j][0]; k++){
            if(pbuf[i] == statarray[j][k+1]) m++;
            i++;
        }
        if(m == statarray[j][0]) result = j;
    }
    
    return result;    
}

/*============================================================================*/

/*
 * コマンドの文字列が含まれているか調べる
 *      cp : check start position in lnbuf[]  
 *      return index of found command 
 *      return 0xFF, if no command was found
 */
unsigned char chkcmd(unsigned char cp){ /* check command in line buffer */
    
    unsigned char i, j, k, m, result;
    
    result = 0xFF;
    for(j=0; j<CMDNUM; j++){
        i = cp;
        m = 0;
        for(k=0; k<cmdarray[j][0]; k++){
            if(lnbuf[i] == cmdarray[j][k+1]) m++;
            i++;
        }
        if(m == cmdarray[j][0]) result = j;
    }
    
    return result;
    
}

/*============================================================================*/


/*
 * コマンドを実行する
 *   "cn":コマンド番号
 *       retrun error code   
 *       0 : no error occured    
 *       2 : invalid operand     
 */
unsigned char execmd(unsigned char cn){ /* execute specified command */
    /* 'cn' is the command index */
    unsigned char errc;
    unsigned int ad_result;
    unsigned char cnum, i;
    unsigned char txstr[20];
    
    errc = 0;   /* default no error */
    if(cn == 16){   /* "RA0?" (no operand is necessary) */
        ADPCH = 0;        // select ANA0
        ADCON0bits.ADGO = 1;    // A/D 変換を開始する //
        while(ADCON0bits.ADGO);
        ad_result = (ADRESH << 8);
        ad_result += ADRESL;
        cnum = sprintf(txstr,"%u (%#x) \r\n", ad_result, ad_result);
        for(i=0; i<cnum; i++){
            while(!TX1STAbits.TRMT);
            TX1REG = txstr[i]; 
        }
    }
    else if(cn == 17){  /* "RA1?" (no operand is necessary) */
        ADPCH = 1;        // select ANA1
        ADCON0bits.ADGO = 1;    // A/D 変換を開始する //
        while(ADCON0bits.ADGO);
        ad_result = (ADRESH << 8);
        ad_result += ADRESL;
        cnum = sprintf(txstr,"%u (%#x) \r\n", ad_result, ad_result);
        for(i=0; i<cnum; i++){
            while(!TX1STAbits.TRMT);
            TX1REG = txstr[i]; 
        }
    }
    else if(cn == 18){  /* "RA2?" (no operand is necessary) */
        //newline();      
        while(!TX1STAbits.TRMT);
        if(PORTAbits.RA2)  TX1REG = '1'; 
        else   TX1REG = '0'; 
        newline();      
    }
    else if(cn == 19){  /* "RB?" (no operand is necessary) */
        //newline();
        i = (PORTB & 0x0F); /* read RB0 -- RB3 */
        while(!TX1STAbits.TRMT);
        TX1REG = '0';
        while(!TX1STAbits.TRMT);
        TX1REG = 'x';
        while(!TX1STAbits.TRMT);
        if(i < 10) TX1REG = 0x30 + i; 
        else TX1REG = 0x37 + i; 
        newline();      
    }
    else if(cn == 20){  /* "RB0?" (no operand is necessary) */
        //newline();      
        while(!TX1STAbits.TRMT);
        if(PORTBbits.RB0)  TX1REG = '1'; 
        else   TX1REG = '0'; 
        newline();      
    }
    else if(cn == 21){  /* "RB1?" (no operand is necessary) */
        //newline();      
        while(!TX1STAbits.TRMT);
        if(PORTBbits.RB1)  TX1REG = '1'; 
        else   TX1REG = '0'; 
        newline();      
    }
    else if(cn == 22){  /* "RB2?" (no operand is necessary) */
        //newline();      
        while(!TX1STAbits.TRMT);
        if(PORTBbits.RB2)  TX1REG = '1'; 
        else   TX1REG = '0'; 
        newline();      
    }
    else if(cn == 23){  /* "RB3?" (no operand is necessary) */
        //newline();      
        while(!TX1STAbits.TRMT);
        if(PORTBbits.RB3)  TX1REG = '1'; 
        else   TX1REG = '0'; 
        newline();      
    }
    else if(cn == 0){   /* "RA=" */
        ci += 3;  /* move to the top of operand */
        if(operand())   LATA = opr;
        else errc = 2;  /* invalid operand */
    }
    else if(cn == 7){   /* "RC=" */
        ci += 3;  /* move to the top of operand */
        if(operand())   LATC = opr;
        else errc = 2;  /* invalid operand */
    }
    else if(cn == 1){   /* "RA3=" */
        ci += 4;  /* move to the top of operand */
        if(operand())   LATAbits.LATA3 = opr;    /* operand must be 0 ro 1 */
        else errc = 2;  /* invalid operand */
    }
    else if(cn == 2){   /* "RA4=" */
        ci += 4;  /* move to the top of operand */
        if(operand())   LATAbits.LATA4 = opr;    /* operand must be 0 ro 1 */
        else errc = 2;  /* invalid operand */
    }
    else if(cn == 3){   /* "RA5=" */
        ci += 4;  /* move to the top of operand */
        if(operand())   LATAbits.LATA5 = opr;    /* operand must be 0 ro 1 */
        else errc = 2;  /* invalid operand */
    }
    else if(cn == 4){   /* "RB2=" */
        ci += 4;  /* move to the top of operand */
        if(operand()){
            if(opr == 0){
                LATBbits.LATB2 = 0;
                sound &= 0xFB;  /* make RB2=0 */
            }
            else{
                LATBbits.LATB2 = 1;
                sound |= 0x04;  /* make RB2=1 */
            }
        }
        else errc = 2;  /* invalid operand */
    }
    else if(cn == 5){   /* "RB3=" */
        ci += 4;  /* move to the top of operand */
        if(operand()){
            if(opr == 0){
                LATBbits.LATB3 = 0;
                sound &= 0xF7;  /* make RB3=0 */
            }
            else{
                LATBbits.LATB3 = 1;
                sound |= 0x08;  /* make RB3=0 */
            }
        }
        else errc = 2;  /* invalid operand */
    }
    else if(cn == 6){  /* "RB5?" (no operand is necessary) */
        //newline();      
        while(!TX1STAbits.TRMT);
        if(PORTBbits.RB5)  TX1REG = '1'; 
        else   TX1REG = '0'; 
        newline();      
    }
    else if(cn == 8){   /* "RC0=" */
        ci += 4;  /* move to the top of operand */
        if(operand())   LATCbits.LATC0 = opr;    /* operand must be 0 ro 1 */
        else errc = 2;  /* invalid operand */
    }
    else if(cn == 9){   /* "RC1=" */
        ci += 4;  /* move to the top of operand */
        if(operand())   LATCbits.LATC1 = opr;    /* operand must be 0 ro 1 */
        else errc = 2;  /* invalid operand */
    }
    else if(cn == 10){   /* "RC2=" */
        ci += 4;  /* move to the top of operand */
        if(operand())   LATCbits.LATC2 = opr;    /* operand must be 0 ro 1 */
        else errc = 2;  /* invalid operand */
    }
    else if(cn == 11){   /* "RC3=" */
        ci += 4;  /* move to the top of operand */
        if(operand())   LATCbits.LATC3 = opr;    /* operand must be 0 ro 1 */
        else errc = 2;  /* invalid operand */
    }
    else if(cn == 12){   /* "RC4=" */
        ci += 4;  /* move to the top of operand */
        if(operand())   LATCbits.LATC4 = opr;    /* operand must be 0 ro 1 */
        else errc = 2;  /* invalid operand */
    }
    else if(cn == 13){   /* "RC5=" */
        ci += 4;  /* move to the top of operand */
        if(operand())   LATCbits.LATC5 = opr;    /* operand must be 0 ro 1 */
        else errc = 2;  /* invalid operand */
    }
    else if(cn == 14){   /* "RC6=" */
        ci += 4;  /* move to the top of operand */
        if(operand())   LATCbits.LATC6 = opr;    /* operand must be 0 ro 1 */
        else errc = 2;  /* invalid operand */
    }
    else if(cn == 15){   /* "RC7=" */
        ci += 4;  /* move to the top of operand */
        if(operand())   LATCbits.LATC7 = opr;    /* operand must be 0 ro 1 */
        else errc = 2;  /* invalid operand */
    }
    else if(cn == 24){  /* "wait" */
        ci += 4;
        if(lnbuf[ci] == '('){
            ci++;
            if(operand()){
                if(opr >= TIMECONST) opr = TIMECONST - 1;
                t2_icnt = 0;
                while(t2_icnt < opr);
            }
            else errc = 2;
        }  
    }
    else if(cn == 25){  /* "SON" (sound ON) */
        TRISB = 0xA3;   // make RB2, 3 output
        sound = 0x0C;   /* RB2, RB3 = 1 */
        T1CONbits.TMR1ON = 1;
        PIR4bits.TMR1IF = 0;
        PIE4bits.TMR1IE= 1;          // TMR1の割り込みを使用        
    }
    else if(cn == 26){  /* "SOFF" (sound OFF) */
        TRISB = 0xAF;   // make RB2, 3 input
        T1CONbits.TMR1ON = 0;
        PIR4bits.TMR1IF = 0;
        PIE4bits.TMR1IE= 0;          // disable TMR1 interrupt        
    }
    else if(cn == 27) do1();    /* lower Do */
    else if(cn == 28) re();    /* Re */
    else if(cn == 29) mi();    /* Mi */
    else if(cn == 30) fa();    /* Fa */
    else if(cn == 31) so();    /* So */
    else if(cn == 32) ra();    /* Ra */
    else if(cn == 33) si();    /* Si */
    else if(cn == 34) do2();    /* upper Do */
    else if(cn == 35){  /* "print" */
        ci += 4;
        if(lnbuf[ci] == 't'){
            ci++;
            if(lnbuf[ci] == '('){
                ci++;
                if(lnbuf[ci] == '"'){
                    ci++;
                    while(lnbuf[ci] != '"'){
                        while(!TX1STAbits.TRMT);
                        TX1REG = lnbuf[ci]; 
                        ci++;
                    }
                }
                else if(operand()){
                    cnum = sprintf(txstr,"%u (%#x) \r\n", opr, opr);
                    for(i=0; i<cnum; i++){
                        while(!TX1STAbits.TRMT);
                        TX1REG = txstr[i]; 
                    }
                }
                else{
                    perr = 0;   /* ignore error flag */
                    newline();
                }
            }
        }  
    }    
    else if(cn == 36){  /* "send" */
        ci += 4;
        if(lnbuf[ci] == '('){
            ci++;
            if(operand()){
                while(!TX1STAbits.TRMT);
                TX1REG = opr; 
            }
            else errc = 2;
        }
    }
    
    return errc;
    
}

/*============================================================================*/


/*
 * コマンドのオペランドの値を求める
 *     戻り値：オペランドとして有効な文字数
 *     オペランドの値は opr に入る
 */
unsigned char operand(void){
    
    unsigned char opflg;    /* valid character number as operand */
    unsigned char w;
    unsigned char eo;  /* end of operand flag */
    
    for(w=0; w<20; w++) sbuf[w] = 0;
    w = 0;
    opflg = 0;
    eo = 0;
    while(!eo){ /* copy the oerand part to sbuf[]  */
        if(lnbuf[ci+w] < 0x20) eo = 1;
        else if(lnbuf[ci+w] == ')') eo = 1;
        else if(lnbuf[ci+w] == 0x20) w++;   /* ignore space */
        else{ sbuf[opflg] = lnbuf[ci+w]; opflg++; w++;}
    }   /* "opflg" is the number of characters in operand */
    opr = evalpara(opflg);
    
    if(perr > 0) opflg = 0;
    
    return opflg;   /* return effective number of characters as operand */ 
    /* effective operand value is in "opr" */
}

/*============================================================================*/

void newline(void){
    
    while(!TX1STAbits.TRMT);
    TX1REG = 0x0d; 
    while(!TX1STAbits.TRMT);
    TX1REG = 0x0a;         
    
}

/*============================================================================*/



void disptitle(void){
    
    unsigned char i;
    
    while(!TX1STAbits.TRMT);
    TX1REG = 0x0d;
    while(!TX1STAbits.TRMT);
    TX1REG = 0x0a;
    for(i=0; i<224; i++){
        while(!TX1STAbits.TRMT);
        TX1REG = title[i];
    }
    while(!TX1STAbits.TRMT);
    TX1REG = 0x0d;
    while(!TX1STAbits.TRMT);
    TX1REG = 0x0a;
    
}

/*============================================================================*/

void prompt(void){
    
    newline();
    while(!TX1STAbits.TRMT);
    TX1REG = '>';
    
}

/*============================================================================*/

void xmtmsg(unsigned char m){
    
    unsigned char i;
    
    newline();
    
    if(m==1){   /* "Invalid Command!" */
        for(i=0; i<16; i++){
            while(!TX1STAbits.TRMT);
            TX1REG = msg1[i];
        }
    }
    else if(m==2){  /* "Invalid Operand!"  */
        for(i=0; i<16; i++){
            while(!TX1STAbits.TRMT);
            TX1REG = msg2[i];
        }
    }
    else if(m==3){  /* "Syntax Error!" */
        for(i=0; i<13; i++){
            while(!TX1STAbits.TRMT);
            TX1REG = msg3[i];
        }
    }
    else if(m==4){  /* "Program Buffer Full!" */
        for(i=0; i<20; i++){
            while(!TX1STAbits.TRMT);
            TX1REG = msg4[i];
        }
    }

    newline();
    
}

/*============================================================================*/


void bin8_to_bcd(unsigned char bin8){

    unsigned char i, j, s;

    for(i=0; i<3; i++) bcd[i] = 0;

    for(j=0; j<8; j++){
        s = 1;
        if(bin8 < 0x80) s = 0;
        bin8 += bin8;		//shift left

        for(i=0; i<3; i++){
            bcd[i] += bcd[i]; 	 //shift left
        }
        bcd[0] += s;
        for(i=0; i<2; i++){
            if(bcd[i] > 15){
                bcd[i] &= 0x0F;
                bcd[i+1]++;
            }
        }
        if(j < 7){
            for(i=0; i<3; i++){
                if(((bcd[i]+3) & 0x08) !=0) bcd[i] += 3;
            }
        }
    }

}
