//
//  新居浜高専ＰＩＣマイコン学習キットVer.3
//      PIC16F18857使用版
//
//      (2021 Nov.)
//      Copyright: 新居浜高専 電子制御工学科 出口幹雄
//
//  * I/O のアサインメント
//      RA0: VR (アナログ入力)
//      RA1: CdS（アナログ入力）
//      RA2: マイク（デジタル入力）
//      RA3: select0　（出力）
//      RA4: select1　（出力）
//      RA5: select2　（出力）
//
//      RB0: SW1（入力）
//      RB1: SW2（入力）
//      RB2: SW3（入力）/charge（出力）
//      RB3: SW4（入力）/discharge（出力）
//      RB4: sound（出力）
//      RB5: タッチセンサ（入力）[タッチアラーム以外のプログラムでは出力に再設定]
//      RB6: PGC(ICSP)
//      RB7: PGD(ICSP)
//
//      RC0-7: LEDのデータ（出力）［ElectricPianoの場合のみ入力］
//
//
//  * LED の選択
//     PORTA に 0x00 ：７セグメントLED左端桁
//     PORTA に 0x08 ：７セグメントLED左側の１桁目
//     PORTA に 0x10 ：７セグメントLED右側の２桁目
//     PORTA に 0x18 ：７セグメントLED右端桁
//     PORTA に 0x20 ：さいころLED
//     PORTA に 0x28 ：フルカラーLED
//     PORTA に 0x30 ：8bitLED
//     PORTA に 0x38 ：LEDの選択無効
//
//
//  MPLABX XC8 フリーモードでビルドしてください
//

/* 水晶8MHz 4xPLL で Fosc = 32MHz にて動作させる場合は、次行をコメントアウトしてください */
//#define XTAL20MHz


// CONFIG1
#pragma config FEXTOSC = HS     // External Oscillator mode selection bits (HS (crystal oscillator) above 4MHz; PFM set to high power)
#ifdef XTAL20MHz
    #pragma config RSTOSC = EXT1X   // Power-up default value for COSC bits (EXTOSC operating per FEXTOSC bits)
#else
    #pragma config RSTOSC = EXT4X   // Power-up default value for COSC bits (EXTOSC with 4x PLL, with EXTOSC operating per FEXTOSC bits)
#endif
#pragma config CLKOUTEN = ON    // Clock Out Enable bit (CLKOUT function is enabled; FOSC/4 clock appears at OSC2)
#pragma config CSWEN = ON       // Clock Switch Enable bit (Writing to NOSC and NDIV is allowed)
#pragma config FCMEN = ON       // Fail-Safe Clock Monitor Enable bit (FSCM timer enabled)

// CONFIG2
#pragma config MCLRE = ON       // Master Clear Enable bit (MCLR pin is Master Clear function)
#pragma config PWRTE = ON       // Power-up Timer Enable bit (PWRT enabled)
#pragma config LPBOREN = OFF    // Low-Power BOR enable bit (ULPBOR disabled)
#pragma config BOREN = ON       // Brown-out reset enable bits (Brown-out Reset Enabled, SBOREN bit is ignored)
#pragma config BORV = LO        // Brown-out Reset Voltage Selection (Brown-out Reset Voltage (VBOR) set to 1.9V on LF, and 2.45V on F Devices)
#pragma config ZCD = OFF        // Zero-cross detect disable (Zero-cross detect circuit is disabled at POR.)
#pragma config PPS1WAY = ON     // Peripheral Pin Select one-way control (The PPSLOCK bit can be cleared and set only once in software)
#pragma config STVREN = ON      // Stack Overflow/Underflow Reset Enable bit (Stack Overflow or Underflow will cause a reset)

// CONFIG3
#pragma config WDTCPS = WDTCPS_31// WDT Period Select bits (Divider ratio 1:65536; software control of WDTPS)
#pragma config WDTE = OFF       // WDT operating mode (WDT Disabled, SWDTEN is ignored)
#pragma config WDTCWS = WDTCWS_7// WDT Window Select bits (window always open (100%); software control; keyed access not required)
#pragma config WDTCCS = SC      // WDT input clock selector (Software Control)

// CONFIG4
#pragma config WRT = OFF        // UserNVM self-write protection bits (Write protection off)
#pragma config SCANE = available// Scanner Enable bit (Scanner module is available for use)
#pragma config LVP = ON         // Low Voltage Programming Enable bit (Low Voltage programming enabled. MCLR/Vpp pin function is MCLR.)

// CONFIG5
#pragma config CP = OFF         // UserNVM Program memory code protection bit (Program Memory code protection disabled)
#pragma config CPD = OFF        // DataNVM code protection bit (Data EEPROM code protection disabled)

//#include <xc.h>
#include <pic16f18857.h>

#include <stdlib.h>
#include <stdio.h>

/* Fosc = 32MHz ---> Fosc/4 = 8MHz (Tcy = 0.125us)  */
/* Fosc = 20MHz ---> Fosc/4 = 5MHz (Tcy = 0.2us)  */


// 時計の１秒の長さを決める定数
#define TIMECONST 4999  // 標準値は 4999   値を大きくすると遅れ、小さくすると進みになる



// 関数のプロトタイプ宣言
// プログラム
void Clock(void);            // #0
void BinaryCount(void);     // #1
void Wave(void);             // #2
void ElectricPiano(void);   // #3
void TouchArarm(void);      // #4
void BellInsect(void);      // #5
void IllusionLight(void);   // #6
void CandleLight(void);     // #7
void Moguratataki(void);    // #8
void Theremin(void);        // #9
void MorningBird(void);     // #10
void Saikoro(void);         // #11
void SoundSwitch(void);     // #12
void ADconverter(void);     // #13
void RCservo(void);         // #14
void Timer(void);           // #15
// サブルーチン
void wait(unsigned char n);
void L_wait(unsigned char n);
void ararm(void);
unsigned char random(void);
void bin16_to_bcd(unsigned int bin16);

void do1(void);
void re(void);
void mi(void);
void fa(void);
void so(void);
void ra(void);
void si(void);
void do2(void);

extern void monitor(void);


// グローバル変数の定義
unsigned char bcd[5];   // bin16_to_bcd()関数の戻り値
unsigned int t2_icnt;   // TMR2割り込み回数のカウンタ
unsigned char sec_flag; // １秒経過のフラグ
unsigned char t1h_0, t1l_0;  // TMR1の初期値
unsigned char sound;    // 音発生用のPORTB出力値
unsigned char segdata[16]={ // ０--F ==> セグメントデータ変換用の配列
    0b11000000, 0b11111001, 0b10100100, 0b10110000,   // 0, 1, 2, 3,
    0b10011001, 0b10010010, 0b10000010, 0b11011000,   // 4, 5, 6, 7,
    0b10000000, 0b10010000, 0b10001000, 0b10000011,   // 8, 9, A, b,
    0b11000110, 0b10100001, 0b10000110, 0b10001110    // C, d, E, F
};

unsigned char rcvflg;
unsigned char rcvdata;
//unsigned char lnbuf[50];


// 割り込み処理
void __interrupt () isr(void){

    if(PIR4bits.TMR1IF){         // 音の発生
        PIR4bits.TMR1IF = 0;
        TMR1H = t1h_0;  // t1h_0, t1l_0 の値が大きいほど、音は高くなる
        TMR1L = t1l_0;  //
        sound ^= 0x10;  // RB4 を0/1入れ替え
        LATB = sound;
    }
    else if(PIR4bits.TMR2IF){    // period 200us
        PIR4bits.TMR2IF = 0;
        t2_icnt++;
        if(t2_icnt > TIMECONST){
            sec_flag = 1;
            t2_icnt = 0;
        }
    }
    else if(PIR3bits.RCIF){
        PIR3bits.RCIF = 0;
        rcvdata = RC1REG;
        rcvflg = 1;
    }

}

// 初期設定 //
void SetConfig(void){
    
    PMD0 = 0x5F;    /* SYSC  FVR   ---   CRC  :  SCAN  NVM   CLKR  IOC  */
    PMD1 = 0xF9;    /* NCO   TMR6  TMR5  TMR4 :  TMR3  TMR2  TMR1  TMR0 */
    PMD2 = 0x47;    /* ---   DAC   ADC   ---  :  ---   CMP2  CMP1  ZCD  */
    PMD3 = 0x7F;    /* ---   PWM7  PWM6  CCP5 :  CCP4  CCP3  CCP2  CCP1 */
    PMD4 = 0x37;    /* ---   UART  MSSP2 MSSP1:  ---   CWG3  CWG2  CWG1 */
    PMD5 = 0xDF;    /* SMT2  SMT1  ---   CLC4 :  CLC3  CLC2  CLC1  DSM  */

    ANSELA  = 0x03;	// RA0,2 をアナログで利用
    ANSELB = 0x00;	// RB はデジタルで利用
    ANSELC = 0x00;	// RC はデジタルで利用
    
// 各ポートで用いる信号の方向（入力／出力）の選択
    TRISA = 0x07;	// RA0,1,2 は入力に設定
    TRISB = 0xAF;	// RB0-3,5,7 は入力に設定
    TRISC = 0x00;	// RC0-7 は出力に設定
    
    WPUB = 0x80;    /* pull-up RB7(RxD) */

// 割り込み設定
//    RBIE  = 0;          // PORTBの割り込みを禁止
//    T0IE  = 0;          // TMR0の割り込みを禁止
    PIE4bits.TMR1IE = 0;          // TMR1の割り込みを禁止
    PIE4bits.TMR2IE = 0;          // TMR2の割り込みを禁止
    INTCONbits.PEIE  = 0;          // 周辺装置の割り込みを禁止

// タイマー設定

    T1CON = 0x02;         // TMR1:0FF, Prescale=1:1 ( 0.2us at Fosc=20MHz, 0.125us at Fosc=32MHz ), RD16=1
    T1GCON = 0x00;
    T1CLK = 0x01;   /* clock is Fosc/4 */
    
    
#ifdef XTAL20MHz    
    T2CLKCON = 0x01;    /* use Fosc/4 as clock source */
    T2CONbits.CKPS = 2; /* prescale 1:4 (count every 0.8us) */
    T2CONbits.OUTPS = 0;    /* poset scale 1:1 */
    T2HLT = 0;  /* free running mode */
    T2RST = 1;
    T2PR = 249;     /* period = 200us */
#else
    T2CLKCON = 0x01;    /* use Fosc/4 as clock source */
    T2CONbits.CKPS = 3; /* prescale 1:8 (count every 1us) */
    T2CONbits.OUTPS = 0;    /* poset scale 1:1 */
    T2HLT = 0;  /* free running mode */
    T2RST = 1;
    T2PR = 199;     /* period = 200us */
#endif    
    T2CONbits.ON = 1;
    

// A/Dコンバータの設定
//    ADCON0 = 0x81;	// Fosc/32, channel 0, A/D : ON
//    ADFM = 1;   // right justified
//    VCFG0 = 0;  // Vref+ = VDD
//    VCFG1 = 0;  // Vref- = VSS
    /* ADC */
    ADCON0 = 0x84;  /* ON, ADCS=0, ADFRM0=1 (right justified) */
    ADCON1 = 0;
    ADCON2 = 0;
    ADCON3 = 0;
    ADCLK = 0x3F;   /* ADC clock is Fosc/128 (TAD=4us) */
    ADREF = 0x00;   /* Vref+ is VDD */
    ADPCH = 0x00;  /* select RA0(ANA0) */

/* UART */    
    RC1STA = 0x90;  /* SPEN=1, CREN=1 */
    TX1STA = 0x24;  /* TXEN=1, BRGH=1 */
    BAUD1CONbits.BRG16 = 1;
#ifdef XTAL20MHz
    SP1BRGH = 0x02; /* SPBRG = 520 for 9600baud */
    SP1BRGL = 0x08;
#else
    SP1BRGH = 0x03; /* SPBRG = 832 for 9600baud */
    SP1BRGL = 0x40;
#endif    
    
    PIR3bits.RCIF = 0;
    PIE3bits.RCIE = 1;
    
    RXPPS = 0x0F;  /* RxD is assigned to RB7 */
    RB6PPS = 0x10;  /* RB6 is tied to TxD */
    
// ポートの初期値    
    LATA = 0x38;       // LED選択無効
    LATC = 0xFF;       //　LEDは消灯
    
}

int main(void){
    
    unsigned char sw;

    
    SetConfig();    // 初期設定

    
    //    プログラムの選択
    // SW4    SW3    SW2    SW1
    //  -      -      -      -     #0 Clock()
    //  -      -      -      ON    #1 BinaryCount()
    //  -      -      ON     -     #2 Wave()
    //  -      -      ON     ON    #3 ElectricPiano()
    //  -      ON     -      -     #4 TouchArarm()
    //  -      ON     -      ON    #5 BellInsect()
    //  -      ON     ON     -     #6 IllusionLight()
    //  -      ON     ON     ON    #7 CandleLight()
    //  ON     -      -      -     #8 Moguratataki()
    //  ON     -      -      ON    #9 Theremin()
    //  ON     -      ON     -     #10 MorningBird()
    //  ON     -      ON     ON    #11 Saikoro()
    //  ON     ON     -      -     #12 SoundSwitch()
    //  ON     ON     -      ON    #13 ADconverter()
    //  ON     ON     ON     -     #14 RCservo()
    //  ON     ON     ON     ON    #15 Timer()
    //

    sw = 0x0F & (~PORTB);

    while((0x0F & PORTB) != 0x0F);  // スイッチを離すまで待つ

    if(sw == 1) BinaryCount();  // #1
    else if(sw == 2) Wave();
    else if(sw == 3) ElectricPiano();
    else if(sw == 4) TouchArarm();
    else if(sw == 5) BellInsect();
    else if(sw == 6) IllusionLight();
    else if(sw == 7) CandleLight();
    else if(sw == 8) Moguratataki();
    else if(sw == 9) Theremin();
    else if(sw == 10) MorningBird();
    else if(sw == 11) Saikoro();
    else if(sw == 12) SoundSwitch();
    else if(sw == 13) ADconverter();
    else if(sw == 14) RCservo();
    else if(sw == 15) Timer();
    else Clock();   // #0

    return 0;
    
}


//=================================================================================================

//////////////////////////////////////////////////////////////////////////////////////
//   #0 Clock
//////////////////////////////////////////////////////////////////////////////////////
//
// 単純なデジタル時計（12時間表示）、７セグメントLEDに時刻を表示
// SW4 を押しながら SW3 で時間の値の設定（１ずつ増加するだけ）
// SW4 を押しながら SW2 で10分の位の値の設定（１ずつ増加するだけ）
// SW4 を押しながら SW1 で1分の位の値の設定（１ずつ増加するだけ）
// SW4/SW1 で1分の位を変更した瞬間に秒のカウントがリセットされる
// 進む時は TIMECONST の値を微増、遅れる時は微減して調節
// さらに微妙な調節をするためには、若干進み気味に TIMECONST を設定した上で、
// OSC2ピンにつながる 15pF のセラミックコンデンサと並列に10pF程度のコンデンサ
// あるいはトリマコンデンサをつないで微調節する
//
// TMR2割り込み（200us周期）5000回毎に sec_falg が 1 になるので、
// それを見て s をカウントアップ。
// s が 60 になったら、時間の値をカウントアップ。
// hour10 : 時間の10の位
// hour1  : 時間の1の位
// min10  : 分の10の位
// min1   : 分の1の位
//
void Clock(void){
    
    unsigned char hour10, hour1, min10, min1, s, kf;
    unsigned char i;

    TRISB = 0x8F;   // RB5を出力に変更

    PIE4bits.TMR2IE = 1;  //TMR2割り込みを使用
    INTCONbits.PEIE = 1;
    INTCONbits.GIE  = 1;
     
    hour10 = 0;
    hour1 = 0;
    min10 = 0;
    min1 = 0;
    s = 0;
    kf = 2;
    sec_flag = 0;
    t2_icnt = 0;
    
    while(1){
        for(i = 0; i < 255; i++){   // ７セグメントLEDをダイナミック点灯
            LATC = 0xFF;
            if(i < 64){
                if(hour10 == 1) LATC = segdata[hour10];    // 時間は１０の位が０の時は点灯しない
                LATA = 0x00;
            }
            else if(i < 128){
                if(t2_icnt < 2500) LATC = 0x7F & segdata[hour1];   // 「：」を１秒毎に点滅
                else LATC = segdata[hour1];
                LATA = 0x08;
            }
            else if(i < 192){
                LATC = segdata[min10];
                LATA = 0x10;
            }
            else{
                LATC = segdata[min1];
                LATA = 0x18;
            }
            if(sec_flag == 1){  // １秒経過時の処理
                sec_flag = 0;
                s++;
                if(s == 60){    // １分経過時の処理
                    s = 0;
                    min1++;
                    if(min1 == 10){
                        min1 = 0;
                        min10++;
                        if(min10 == 6){
                            min10 = 0;
                            hour1++;
                            if(hour10 == 0){
                                if(hour1 == 10){
                                    hour1 = 0;
                                    hour10++;
                                }
                            }
                            else{
                                if(hour1 == 2){     // case 12:00
                                    hour10 = 0;
                                    hour1 = 0;
                                    min10 = 0;
                                    min1 = 0;
                                }
                            }
                        }

                    }
                }

            }
        }
        if((PORTB  & 0x0F) == 0x07) kf = 0;
        if((PORTB  & 0x0F) > 0x07) kf = 2;    // make key inactive
        if(kf == 0){    // SW4を押している場合
            if(PORTBbits.RB2 == 0){ // SW3も押している場合
                kf = 1;
                hour1++;
                if(hour10 == 0){
                    if(hour1 == 10){
                        hour1 = 0;
                        hour10++;
                    }
                }
                else{   // case hour10 = 1
                    if(hour1 == 2){
                        hour1 = 0;
                        hour10 = 0;
                    }
                }
            }
            if(PORTBbits.RB1 == 0){ // SW2も押している場合
                kf = 1;
                min10++;
                if(min10 == 6) min10 = 0;
            }
            if(PORTBbits.RB0 == 0){ // SW1も押している場合
                kf = 1;
                min1++;
                if(min1 == 10) min1 = 0;
                s = 0;
                t2_icnt = 0;
            }
        }
        if((PORTB & 0x07)==0) monitor();    /* when SW1, SW2, SW3 are all ON, go to monitor program */
    }
      
}



//////////////////////////////////////////////////////////////////////////////////////
//  #1  Binary Count
//////////////////////////////////////////////////////////////////////////////////////
//
// a : カウンタ
// SW1で a をカウントアップ
// SW2で a をカウントダウン
// SW3で a を0クリア
// SW4で a を１秒毎にカウントアップのスタート／ストップ
// a の値を8bitLEDで２進数表示、７セグメントLED（右側）で１６進数表示
//
void BinaryCount(void){

    unsigned char a, b, c;
    unsigned char key_st, status;

        TRISB = 0x8F;   // RB5を出力に変更
        PIE4bits.TMR2IE = 1;  //TMR2割り込みを使用
        INTCONbits.PEIE = 1;
        INTCONbits.GIE = 1;

        a = 0;
        c = 0;
        key_st = 0;
        status = 0;
        sec_flag = 0;

        while(1){

            b = 0x0F & PORTB;   // read SW
            if(b == 0x0F)  key_st = 0;   // no SW is pressed
            
            if(key_st == 0){
                if(b == 0x0E){  // if SW1 is pressed
                    a++;
                    key_st = 1;
                }
                if(b == 0x0D){  // if SW2 is pressed
                    a--;
                    key_st = 1;
                }
                if(b == 0x0B){  // if SW3 is pressed
                    a = 0;
                    key_st = 1;
                }
                if(b == 0x07){  // if SW4 is pressed
                    status ^= 1;
                    key_st = 1;
                }
            }

            if(status == 1){    // auto count up mode
                if(sec_flag == 1){  // when 1 second passed
                    sec_flag = 0;
                    a++;
                }
            }

            LATC = 0xFF;
            if(c == 0){
                LATA = 0x30;   // select 8-LEDs
                LATC = ~a;
                c++;
            }
            else if(c == 1){
                LATA = 0x18;   // select 7seg-LED0
                b = 0x0F & a;   // extract lower 4-bit
                LATC = segdata[b];
                c++;
            }
            else{
                LATA = 0x10;   // select 7seg-LED1
                b = (0xF0 & a) >> 4;    // extract higher 4-bit
                LATC = segdata[b];
                c = 0;
            }
            wait(100);
        }
    
}


//////////////////////////////////////////////////////////////////////////////////////
//   #2 Wave
//////////////////////////////////////////////////////////////////////////////////////
//
// duty[n] : 8bitLEDの右から n 番目（n = 0 -- 7）のduty比
// direction = 0 の時、duty[n]をduty[n-1]にシフト
// direction = 1 の時、duty[n]をduty[n+1]にシフト
// duty比はsin波状に変化させる
// direction を変更するまでのシフト回数はランダム
//
void Wave(void){
    
    unsigned char i, j, k, m, n, d, direction;
    unsigned char duty[8];  // 各LEDのduty比
    unsigned char sine[8]={0, 37, 128, 218, 255, 218, 128, 37};

    TRISB = 0x8F;  // RB5を出力に変更
    LATA = 0x30;  // select 8-LEDs
    direction = 0;
    
    while(1){

        m = 0x1F & random();    // 向きを変えるまでの周期はランダム
        for(n = 0; n < m; n++){
            for(j = 0; j < 10; j++){    // PWMのパルスを１０回出す
                for(i = 0; i < 255; i++){ // PWMの処理
                    d = 0xFF;
                    if(i <= duty[0]) d &= 0xFE;
                    if(i <= duty[1]) d &= 0xFD;
                    if(i <= duty[2]) d &= 0xFB;
                    if(i <= duty[3]) d &= 0xF7;
                    if(i <= duty[4]) d &= 0xEF;
                    if(i <= duty[5]) d &= 0xDF;
                    if(i <= duty[6]) d &= 0xBF;
                    if(i <= duty[7]) d &= 0x7F;
                    LATC = d;
                }
            }
            if(direction == 0){
                for(i = 0; i < 7; i++){ // dutyのデータを右にシフト
                    duty[i] = duty[i+1];
                }
                duty[7] = sine[k];
            }
            else{
                for(i = 7; i > 0; i--){ // dutyのデータを左にシフト
                    duty[i] = duty[i-1];
                }
                duty[0] = sine[k];                
            }
            k++;
            if(k > 7) k = 0;
        }
        if(random() > 128) direction = 0; // 向きをランダムに決める
        else direction = 1;
    }
    
}


//////////////////////////////////////////////////////////////////////////////////////
//   #3 Electric Piano
//////////////////////////////////////////////////////////////////////////////////////
//
// このプログラムのみ PORTC を入力として使う
// TMR1割り込み毎に RB4 の H/L を入れ替えて音信号を作る
// t1h_0 : TMR1初期値の上位バイト
// t1l_0 : TMR1初期値の下位バイト
// TMR1の初期値を大きくするほど、割り込み周期が短くなるので音が高くなる
// RB2 を L にすると、圧電ブザーを駆動するトランジスタのコレクタ電圧が充電される
// RB3 を L にすると、コレクタ電圧を一定の時定数で放電するので、ピアノライクな減衰音になる
//
void ElectricPiano(void){

    //　Fosc = 20MHz の場合
    //     f[Hz]   5MHz/f   ÷2=x  65536-x   t1h_0   t1l_0
    // do1 261.6   19113    9557    55979    0xDA    0xAB
    // re  293.7   17024    8512    57024    0xDE    0xC0
    // mi  329.6   15170    7585    57951    0xE2    0x5F
    // fa  349.2   14318    7159    58377    0xE4    0x09
    // so  392.0   12755    6378    59158    0xE7    0x16
    // ra  440     11364    5682    59854    0xE9    0xCE
    // si  493.9   10124    5062    60474    0xEC    0x3A
    // do2 523.3    9555    4777    60759    0xED    0x57
    //
    //　Fosc = 32MHz の場合
    //     f[Hz]   8MHz/f   ÷2=x  65536-x   t1h_0   t1l_0
    // do1 261.6   30581   15291    50245    0xC4    0x45
    // re  293.7   27234   13619    51917    0xCA    0xCD
    // mi  329.6   24272   12136    53400    0xD0    0x98
    // fa  349.2   22910   11455    54081    0xD3    0x41
    // so  392.0   20408   10204    55332    0xD8    0x24
    // ra  440     18182    9091    56445    0xDC    0x7D
    // si  493.9   16198    8099    57437    0xE0    0x5D
    // do2 523.3   15288    7644    57892    0xE2    0x24

    TRISB = 0x83;   // make RB2, 3, 5 output
    TRISC = 0xFF;   // make PORTC all input

    T1CONbits.TMR1ON = 1;

    PIR4bits.TMR1IF = 0;
    PIE4bits.TMR1IE= 1;          // TMR1の割り込みを使用
    INTCONbits.PEIE  = 1;          // 周辺装置（TMR1)の割り込みを許可
    INTCONbits.GIE = 1;

    sound = 0x04;   // TMR1割り込み毎に、sound の値が PORTB に出力され、4bit目が 0/1 入れ替わる

    while(1){

        if(PORTCbits.RC0 == 0){ // do2
            L_wait(100);
            if(PORTCbits.RC0 == 0){
//#ifdef  XTAL20MHz
//                t1h_0 = 0xED;
//                t1l_0 = 0x57;
//#else
//                t1h_0 = 0xE2;
//                t1l_0 = 0x24;                
//#endif                
//                sound = 0x08;   // charge
//                L_wait(10);
//                sound = 0x04;   // discharge
                do2();
                while(PORTCbits.RC0 == 0);
            }
        }
        if(PORTCbits.RC1 == 0){ // si
            L_wait(100);
            if(PORTCbits.RC1 == 0){
//#ifdef  XTAL20MHz
//                t1h_0 = 0xEC;
//                t1l_0 = 0x3A;
//#else
//                t1h_0 = 0xE0;
//                t1l_0 = 0x5D;
//#endif                        
//                sound = 0x08;   // charge
//                L_wait(10);
//                sound = 0x04;   // discharge
                si();
                while(PORTCbits.RC1 == 0);
            }
        }
        if(PORTCbits.RC2 == 0){ // ra
            L_wait(100);
            if(PORTCbits.RC2 == 0){
//#ifdef  XTAL20MHz
//                t1h_0 = 0xE9;
//                t1l_0 = 0xCE;
//#else
//                t1h_0 = 0xDC;
//                t1l_0 = 0x7D;
//#endif                        
//                sound = 0x08;   // charge
//                L_wait(10);
//                sound = 0x04;   // discharge
                ra();
                while(PORTCbits.RC2 == 0);
            }
        }
        if(PORTCbits.RC3 == 0){ // so
            L_wait(100);
            if(PORTCbits.RC3 == 0){
//#ifdef  XTAL20MHz
//                t1h_0 = 0xE7;
//                t1l_0 = 0x16;
//#else
//                t1h_0 = 0xD8;
//                t1l_0 = 0x24;
//#endif                        
//                sound = 0x08;   // charge
//                L_wait(10);
//                sound = 0x04;   // discharge
                so();
                while(PORTCbits.RC3 == 0);
            }
        }
        if(PORTCbits.RC4 == 0){ // fa
            L_wait(100);
            if(PORTCbits.RC4 == 0){
//#ifdef  XTAL20MHz
//                t1h_0 = 0xE4;
//                t1l_0 = 0xD9;
//#else
//                t1h_0 = 0xD3;
//                t1l_0 = 0x41;
//#endif                        
//                sound = 0x08;   // charge
//                L_wait(10);
//                sound = 0x04;   // discharge
                fa();
                while(PORTCbits.RC4 == 0);
            }
        }
        if(PORTCbits.RC5 == 0){ // mi
            L_wait(100);
            if(PORTCbits.RC5 == 0){
//#ifdef  XTAL20MHz
//                t1h_0 = 0xE2;
//                t1l_0 = 0x5F;
//#else
//                t1h_0 = 0xD0;
//                t1l_0 = 0x98;
//#endif                        
//                sound = 0x08;   // charge
//                L_wait(10);
//                sound = 0x04;   // discharge
                mi();
                while(PORTCbits.RC5 == 0);
            }
        }
        if(PORTCbits.RC6 == 0){ //re
            L_wait(100);
            if(PORTCbits.RC6 == 0){
//#ifdef  XTAL20MHz
//                t1h_0 = 0xDE;
//                t1l_0 = 0xC0;
//#else
//                t1h_0 = 0xCA;
//                t1l_0 = 0xCD;
//#endif                        
//                sound = 0x08;   // charge
//                L_wait(10);
//                sound = 0x04;   // discharge
                re();
                while(PORTCbits.RC6 == 0);
            }
        }
        if(PORTCbits.RC7 == 0){ //do1
            L_wait(100);
            if(PORTCbits.RC7 == 0){
//#ifdef  XTAL20MHz
//                t1h_0 = 0xDA;
//                t1l_0 = 0xAB;
//#else
//                t1h_0 = 0xC4;
//                t1l_0 = 0x45;
//#endif                        
//                sound = 0x08;   // charge
//                L_wait(10);
//                sound = 0x04;   // discharge
                do1();
                while(PORTCbits.RC7 == 0);
            }
        }
        L_wait(100);
    }

}


void do2(void){
#ifdef  XTAL20MHz
                t1h_0 = 0xED;
                t1l_0 = 0x57;
#else
                t1h_0 = 0xE2;
                t1l_0 = 0x24;                
#endif                
                sound = 0x08;   // charge
                L_wait(10);
                sound = 0x04;   // discharge    
}

void si(void){
#ifdef  XTAL20MHz
                t1h_0 = 0xEC;
                t1l_0 = 0x3A;
#else
                t1h_0 = 0xE0;
                t1l_0 = 0x5D;
#endif                        
                sound = 0x08;   // charge
                L_wait(10);
                sound = 0x04;   // discharge    
}

void ra(void){
#ifdef  XTAL20MHz
                t1h_0 = 0xE9;
                t1l_0 = 0xCE;
#else
                t1h_0 = 0xDC;
                t1l_0 = 0x7D;
#endif                        
                sound = 0x08;   // charge
                L_wait(10);
                sound = 0x04;   // discharge    
}

void so(void){
#ifdef  XTAL20MHz
                t1h_0 = 0xE7;
                t1l_0 = 0x16;
#else
                t1h_0 = 0xD8;
                t1l_0 = 0x24;
#endif                        
                sound = 0x08;   // charge
                L_wait(10);
                sound = 0x04;   // discharge    
}

void fa(void){
#ifdef  XTAL20MHz
                t1h_0 = 0xE4;
                t1l_0 = 0xD9;
#else
                t1h_0 = 0xD3;
                t1l_0 = 0x41;
#endif                        
                sound = 0x08;   // charge
                L_wait(10);
                sound = 0x04;   // discharge    
}

void mi(void){
#ifdef  XTAL20MHz
                t1h_0 = 0xE2;
                t1l_0 = 0x5F;
#else
                t1h_0 = 0xD0;
                t1l_0 = 0x98;
#endif                        
                sound = 0x08;   // charge
                L_wait(10);
                sound = 0x04;   // discharge    
}

void re(void){
#ifdef  XTAL20MHz
                t1h_0 = 0xDE;
                t1l_0 = 0xC0;
#else
                t1h_0 = 0xCA;
                t1l_0 = 0xCD;
#endif                        
                sound = 0x08;   // charge
                L_wait(10);
                sound = 0x04;   // discharge    
}

void do1(void){
#ifdef  XTAL20MHz
                t1h_0 = 0xDA;
                t1l_0 = 0xAB;
#else
                t1h_0 = 0xC4;
                t1l_0 = 0x45;
#endif                        
                sound = 0x08;   // charge
                L_wait(10);
                sound = 0x04;   // discharge    
}

//////////////////////////////////////////////////////////////////////////////////////
//   #4 Touch Ararm
//////////////////////////////////////////////////////////////////////////////////////
//
// RB5 が L になったらアラームを鳴らす
// プルアップ抵抗が高抵抗のため、電位が中途半端になりやすいので、
// このプログラム以外では、RB5 は出力に設定しておく
//
void TouchArarm(void){

    TRISB = 0xAF;   // RB5は入力

    while(PORTBbits.RB5 == 1);  // RB5＝Lとなったらアラームを鳴らす
    ararm();

}



//////////////////////////////////////////////////////////////////////////////////////
//   #5 Bell Insect
//////////////////////////////////////////////////////////////////////////////////////
//
// RB2 を L とし、圧電ブザー駆動トランジスタのコレクタ電圧を充電した後そのままにしておくと、
// 圧電ブザーの駆動によってじわじわと放電するので、時定数の長い減衰音になる
// 周期の異なる振動を繰り返すことでビブラートをかける
// 少し音を低めにして、コレクタ電圧を充電したまま持続音ににするとコオロギライクな音になる
// 暗くなるとCdSの抵抗が大きくなるので、RA1 が H となる
// すず虫音は７回鳴って８回分の長さ休憩
// 休憩中に暗くなると、コオロギ音に変わる
// コオロギ音が休憩中に暗くなると、すず虫音に戻る
//
void BellInsect(void){

    unsigned char a, b, c;
    unsigned char status;


        ANSELA = 0x01;   // make RA1 digital
        TRISB = 0x83;   // make RB2,3 output
        status = 0;

        while(1){
            if(status == 0){  // Bell insect
                for(a = 0; a < 15; a++){
                    if(a < 7){  // ring 7 times
                        LATB = 0x08;   // charge
                        L_wait(10);
                        LATB = 0x0C;
                    }
                    for(b = 0; b < 70; b++){ // ビブラート７０回
                        for(c = 0; c < 100; c++){ // 高い方の音（振動１００回）
                            LATB = 0x1C;
                            wait(30);
                            LATB = 0x0C;
                            wait(30);
                        }
                        for(c = 0; c < 100; c++){ // 低い方の音（振動１００回）
                            LATB = 0x1C;
                            wait(45);
                            LATB = 0x0C;
                            wait(45);
                        }
                    }
                }
                if(PORTAbits.RA1 == 1) status = 1;  // 暗くなっていたらコオロギ音に変更
            }
            if(status == 1){    // Cricket
                for(a = 0; a < 8; a++){
                    for(b = 0; b < 10; b++){ // ビブラート１０回
                        for(c = 0; c < 100; c++){ // 高い方の音（振動１００回）
                            LATB = 0x18;
                            wait(32);
                            LATB = 0x08;
                            wait(32);
                        }
                        for(c = 0; c < 100; c++){ // 低い方の音（振動１００回）
                            LATB = 0x18;
                            wait(48);
                            LATB = 0x08;
                            wait(48);
                        }
                    }
                    L_wait(100) ;
                }
                for(a = 0; a < 100; a++) L_wait(250);
                if(PORTAbits.RA1 == 1) status = 0;  // 暗くなっていたらすず虫音に変更
            }

        }

}




//////////////////////////////////////////////////////////////////////////////////////
//   #6 Illusion light
//////////////////////////////////////////////////////////////////////////////////////
//
// フルカラーLEDの R, G, B のduty比をランダムに増減する
// 光を乱反射するランプシェード（例えば、サランラップをクシャクシャにしたものなど）をかぶせると雰囲気が出る
//
//
void IllusionLight(void){
    
    unsigned char R_duty, G_duty, B_duty;
    unsigned char R_ip, G_ip, B_ip;
    unsigned char R_f, G_f, B_f;
    unsigned char i, j, d;

    TRISB = 0x8F;    // RB5を出力に変更
    LATA = 0x28;   // select full color LED
    R_duty = random();  // R duty
    G_duty = random();  // G duty
    B_duty = random();  // B duty
    R_ip = 0x0F & random();    // R duty increment period
    G_ip = 0x0F & random();    // G duty increment period
    B_ip = 0x0F & random();    // B duty increment period
    R_f = 0;  // R duty inc/dec flag  0: increment, 1: decrement
    G_f = 0;  // G duty inc/dec flag
    B_f = 0;  // B duty inc/dec flag

    i = 0;

    while(1){

        d = 7;  // data to be output to PORTC
        for(j = 0; j < 255; j++){
            if(j <= R_duty) d &= 0xFE;  // R on
            else d |= 1;    // R off
            if(j <= G_duty) d &= 0xFD;  // G on
            else d |= 2;    // G off
            if(j <= B_duty) d &= 0xFB;  // B on
            else d |= 4;    // B off
            LATC = d;
        }

        if(i >= R_ip){  // if it is time to change duty,
            if(R_f == 0){ // 0 means increase duty
                if(R_duty < 255) R_duty++;  // if not yet full duty, increase duty.
                else R_f = 1;   // if reach full duty, change to decrement.
            }
            else{   // case of decrease duty
                if(R_duty > 0){ // if duty is not zero,
                    R_duty--;   // decrease duty.
                }
                else{
                    R_f = 0;    // if duty has become zero, change to increment.
                    R_ip = 0x0F & random(); // and change the period of duty inc/dec
                }
            }
        }
        if(i >= G_ip){
            if(G_f == 0){
                if(G_duty < 255) G_duty++;
                else G_f = 1;
            }
            else{
                if(G_duty > 0){
                    G_duty--;
                }
                else{
                    G_f = 0;
                    G_ip = 0x0F & random();
               }
            }
        }
        if(i >= B_ip){
            if(B_f == 0){
                if(B_duty < 255) B_duty++;
                else B_f = 1;
            }
            else{
                if(B_duty > 0){
                    B_duty--;
                }
                else{
                    B_f = 0;
                    B_ip = 0x0F & random();
                }
            }
        }

        i++;    // count up the couter
        i &= 0x0F;
    }

}

//////////////////////////////////////////////////////////////////////////////////////
//   #7 Candle light
//////////////////////////////////////////////////////////////////////////////////////
//
// フルカラーLEDを黄色に点灯し、duty比を擬似的な「1/fゆらぎ」で変化させる
// 光をボーッと拡散するランプシェードをかぶせると雰囲気が出る
// duty比を時間に対して直線的に変化させ、増加／減少を切り替えるまでの時間をランダムに決めることで
// 擬似的な1/fゆらぎを作る
// 時々、dutyに急激な変化を与え、炎が風でゆらめく効果を模擬する
//
void CandleLight(void){
    
    unsigned char i,j,k;
    unsigned char duty, flag, priod;

    TRISB = 0x8F;   // RB5を出力に変更
    LATA = 0x28;   // select full color LED
    i = 0;
    duty = 128; // duty の初期値
    priod = 0x0F & random();    // duty の増減を切り替えるまでの周期
    flag = 0;
    while(1){

            for(j = 0; j < 20; j++){    // PWMのパルスを２０個
                for(k = 0; k < 255; k++){   // PWMの処理
                    if(k <= duty) LATC = 0xFC;  // R,G on（黄色）
                    else LATC = 0xFF;    // R,G off
                }
            }

            i++;
            if(i >= priod){ // duty の増減を切り替える時がきたか
                i = 0;
                priod = 0x0F & random();    // 次に切り替えるまでの時間はランダム
                if((duty > 0) && (duty < 255)) flag ^= 1;
            }

            if(flag == 0){  // dutyを増加させる場合
                duty++;
                if(duty == 255) flag = 1;
            }
            else{   // dutyを減少させる場合
                duty--;
                if(duty == 0) flag = 0;
            }

            if(random() == 0) duty -= 50;   // 時々dutyを急激に変化させる

    }

}





//////////////////////////////////////////////////////////////////////////////////////
//   #8 Moguratataki
//////////////////////////////////////////////////////////////////////////////////////
//
// 8bitLEDのどれかが１つ光る
// 光ったbitの番号（0 -- 7）を３桁の２進数で表した時 1 になる桁を SW3, SW2, SW1 を押して答える
// 右端（bit 0 ）が光った時は、SW4 を押す
// スイッチは 0.3秒以上押さえる
// 正解なら「ピン・ポン」、間違えると「ブーー」が鳴る
// 正解数が７セグメントLED（右側）に表示される
//
void Moguratataki(void){

    unsigned char a, b, n, status, i, tm, bell;
    unsigned char p10, p1;

    TRISB = 0x8F;  // RB5を出力に変更
    PIE4bits.TMR2IE = 1;   // TMR2割り込みを使用
    T1CONbits.TMR1ON = 1;
    PIE4bits.TMR1IE= 1;          // TMR1割り込みを使用
    INTCONbits.PEIE  = 1;          // 周辺装置（TMR1)の割り込みを許可
    INTCONbits.GIE  = 1;

    sec_flag = 0;
    i = 0;
    status = 0; // 0: initiallize, 1: wait , 2: MOGURA appears, 3: SW pressed
    p10 = 0;    // 正解数の１０の位
    p1 = 0;     // 正解数の１の位
    random();


    while(1){

        if(status == 0){    // initiallize
            n = (random() >> 5);
            b = 1;
            if(n > 0) b <<= n;
            status = 1;
            sec_flag = 0;   // start time count
            t2_icnt = 0;
            tm = 0;
        }
        else if(status == 1){
            if(sec_flag == 1){
                tm++;   // time count
                sec_flag = 0;
            }
            if(tm == 2){    // ２秒待ってから光らせる
                tm = 0;
                status = 2;
            }
        }
        else if(status == 2){
            if(sec_flag == 1){
                tm++;   // time count
                sec_flag = 0;
            }
            a = 0x0F & PORTB;
            if((tm < 5) && (a != 0x0F)){
                t2_icnt = 0;
                status = 3;
            }
            if(tm >= 5) status = 0; // next
        }
        else if(status == 3){
            if(t2_icnt > 1500){     // SW must be pressed for more than 0.3s
                a = 0x0F & PORTB;
                if(a != 0x0F){
                    if((0x0F & ~a) == n){
                        status = 4; // correct
                    }
                    else if((n == 0) && (a == 0x7)){
                        status = 4; // correct
                    }
                    else{
                        status = 6;    // wrong
                    }
                }
                else{
                    status = 6;
                }
            }
        }
        else if(status == 4){   // 正解の場合
            a = 0x0F & PORTB;
            if(a == 0x0F){  // スイッチが離されるまで待つ
                status = 5; // if SW is released
                bell = 0;
            }
        }
        else if(status == 5){  // 正解してスイッチが離された場合
            if(bell == 0){
                p1++;   // ポイントをカウントアップ
                if(p1 == 10){
                    p1= 0;
                    p10++;
                }
                TRISB = 0x83;   // make RB2,3 output（ピン....）
                // si
#ifdef  XTAL20MHz
                t1h_0 = 0xEC;
                t1l_0 = 0x3A;
#else
                t1h_0 = 0xE0;
                t1l_0 = 0x5D;
#endif                        
                T1CONbits.TMR1ON = 1;
                sound = 0x08;   // charge
                L_wait(10);
                sound = 0x04;   // discharge
                bell = 1;
                t2_icnt = 0;
            }
            if((bell == 1) && (t2_icnt > 2500)){    // （ポン....）
                // so
#ifdef  XTAL20MHz
                t1h_0 = 0xE7;
                t1l_0 = 0x16;
#else
                t1h_0 = 0xD8;
                t1l_0 = 0x24;
#endif                        
                sound = 0x08;   // charge
                L_wait(10);
                sound = 0x04;   // discharge
                bell = 2;
                t2_icnt = 0;
            }
            if((bell == 2) && (t2_icnt > 2500)){
                status = 0;     // 元に戻る
                T1CONbits.TMR1ON = 0;
                TRISB = 0x8F;   // make RB2,3 input
            }
        }
        else if(status == 6){   // 不正解の場合
            a = 0x0F & PORTB;
            if(a == 0x0F){  // if SW is released
                status = 7;
                bell = 0;
            }
        }
        else if(status == 7){  // 不正解でスイッチが離された場合
            if(bell == 0){
                TRISB = 0x83;   // make RB2,3 output（ブーー）
                //do1
#ifdef  XTAL20MHz
                t1h_0 = 0xDA;
                t1l_0 = 0xAB;
#else
                t1h_0 = 0xC4;
                t1l_0 = 0x45;
#endif                        
                T1CONbits.TMR1ON = 1;
                sound = 0x08;   // charge
                t2_icnt = 0;
                bell = 1;
            }
            if(bell == 1){
                if(t2_icnt > 4990){
                    T1CONbits.TMR1ON = 0;
                    TRISB = 0x8F;   // make RB2,3 input
                    sound = 0x0C;
                    status = 0;
                }
            }

        }


        //　７セグメントLED（右側）と8bitLEDをダイナミック点灯
        if(i <= 85){
            LATC = 0xFF;
            LATA = 0x10;   // select 7-segLED1
            LATC = segdata[p10];
            i++;
        }
        else if(i <= 170){
            LATC = 0xFF;
            LATA = 0x18;   // select 7-segLED0
            LATC = segdata[p1];
            i++;
        }
        else{
            LATC = 0xFF;
            LATA = 0x30;   // select 8-LEDs
            if(status >= 2) LATC = ~b;  // MOGURA appears
            else LATC = 0xFF;
            i++;
        }

    }

}



//////////////////////////////////////////////////////////////////////////////////////
//   #9 Theremin
//////////////////////////////////////////////////////////////////////////////////////
//
// CdSの電圧レベルに応じてTMR1の初期値を変え、音の高さを変化させる
// 暗くするほどCdSの抵抗が大きくなるので電圧レベルがあがり、音が高くなる
// 同時に、8bitLEDで電圧レベルをバーグラフ表示する
//
//
void Theremin(void){

    unsigned int a;

    TRISB = 0x83;   // make RB2,3,5 output

    T1CONbits.TMR1ON = 1;
    PIR4bits.TMR1IF = 0;
    PIE4bits.TMR1IE= 1;          // TMR1割り込みを使用
    INTCONbits.PEIE  = 1;          // 周辺装置（TMR1)の割り込みを許可
    INTCONbits.GIE = 1;

    sound = 0x08;   // charge（コレクタ電圧を供給）

    LATA = 0x30;   // select 8-LEDs

    while(1){

        ADPCH = 1;        // select ANA1 (CdS)
        ADCON0bits.ADGO = 1;
        while(ADCON0bits.ADGO);
        a = ADRESH << 8;
        a += ADRESL;

        if(a < 200) PORTC = 0xFF;   // CdSの電圧レベルに応じてバーグラフ表示
        else if(a < 280) PORTC = 0xFE;
        else if(a < 360) PORTC = 0xFC;
        else if(a < 440) PORTC = 0xF8;
        else if(a < 520) PORTC = 0xF0;
        else if(a < 600) PORTC = 0xE0;
        else if(a < 680) PORTC = 0xC0;
        else if(a < 860) PORTC = 0x80;
        else PORTC = 0x00;
        PORTA = 0x30;

        a <<= 3;
        a += 0xE000;

        t1h_0 = a >> 8;     // CdSの電圧レベルに応じて音の高さを変える
        t1l_0 = a;

    }

}


//////////////////////////////////////////////////////////////////////////////////////
//   #10 MorningBird
//////////////////////////////////////////////////////////////////////////////////////
//
// 一旦暗くなって（夜になって）から、明るくなる（朝が来る）とピヨピヨ音が鳴り始める
// ピヨピヨ音が鳴る間隔はランダム
// CdSの電圧はアナログ値で読み取り、暗くなったと判断するレベルと
// 明るくなったと判断するレベルに差をつけることで、ヒステリシスを持たせている
//
void MorningBird(void){

    unsigned int ad;
    unsigned char a, b, c;

    TRISB = 0x8B;   // make RB2 output
    PIE4bits.TMR2IE = 1;   // TMR2割り込みを使用
    INTCONbits.PEIE = 1;   //
    INTCONbits.GIE = 1;

    ad = 0;

    while(ad < 600){    // wait darkness
        ADPCH = 1;        // select ANA1 (CdS)
        ADCON0bits.ADGO = 1;
        while(ADCON0bits.ADGO);
        ad = ADRESH << 8;
        ad += ADRESL;
    }
    while(ad > 300){    // wait brightness
        ADPCH = 1;        // select ANA1 (CdS)
        ADCON0bits.ADGO = 1;
        while(ADCON0bits.ADGO);
        ad = ADRESH << 8;
        ad += ADRESL;
    }

     while(1){

        for(b = 0; b < 4; b++){   // ピヨピヨピヨピヨ
            for(c = 0; c < 150; c++){   // 高い音（振動１５０回）
                LATB = 0x18;
                wait(27);
                LATB = 0x08;
                wait(27);
            }
            for(c = 0; c < 150; c++){
                LATB = 0x18;
                wait(30);
                LATB = 0x08;
                wait(30);
            }
            for(c = 0; c < 150; c++){
                LATB = 0x18;
                wait(33);
                LATB = 0x08;
                wait(33);
            }
            for(c = 0; c < 150; c++){
                LATB = 0x18;
                wait(34);
                LATB = 0x08;
                wait(34);
            }
            for(c = 0; c < 150; c++){   // 低い音（振動１５０回）
                LATB = 0x18;
                wait(37);
                LATB = 0x08;
                wait(37);
            }
        }

        a = 0;
        b = 0x0F & random();   // 鳴く間隔をランダムに決める
        b += 3;   // 間隔は最低３秒
        while(a < b){
            if(sec_flag == 1){
                sec_flag = 0;
                a++;
            }
        }


    }
}


//////////////////////////////////////////////////////////////////////////////////////
//   #11 Denshi Saikoro
//////////////////////////////////////////////////////////////////////////////////////
//
// ７セグメントLEDを８の字ダンス状に光らせてルーレット表示
// SW1を押すと、その瞬間の値がサイコロの目で表示される
// 訓練すれば「目押し」ができるようになり、動体視力を養うことができるかも...
//
void Saikoro(void){

    unsigned char i;

    TRISB = 0x8F;
    i = 0;

    while(1){
        while(i==0){   // ルーレット表示
            LATC = 0xFE;   //a
            LATA = 0x00;
            L_wait(10);
            LATC = 0xFE;   //a
            LATA = 0x18;
            L_wait(10);
            if((PORTBbits.RB0 == 0) && (i == 0)) i = 1;

            LATC = 0xFD;   //b
            LATA = 0x00;
            L_wait(10);
            LATC = 0xDF;   //f
            LATA = 0x18;
            L_wait(10);
            if((PORTBbits.RB0 == 0) && (i == 0)) i = 2;

            LATC = 0xEF;   //e
            LATA = 0x08;
            L_wait(10);
            LATC = 0xFB;    //c
            LATA = 0x10;
            L_wait(10);
            if((PORTBbits.RB0 == 0) && (i == 0)) i = 3;

            LATC = 0xF7;   //d
            LATA = 0x08;
            L_wait(10);
            LATC = 0xF7;   //d
            LATA = 0x10;
            L_wait(10);
            if((PORTBbits.RB0 == 0) && (i == 0)) i = 4;

            LATC = 0xFB;   //c
            LATA = 0x08;
            L_wait(10);
            LATC = 0xEF;    //e
            LATA = 0x10;
            L_wait(10);
            if((PORTBbits.RB0 == 0) && (i == 0)) i = 5;

            LATC = 0xFD;   //b
            LATA = 0x08;
            L_wait(10);
            LATC = 0xDF;    //f
            LATA = 0x10;
            L_wait(10);
            if((PORTBbits.RB0 == 0) && (i == 0)) i = 6;

            LATC = 0xFE;   //a
            LATA = 0x08;
            L_wait(10);
            LATC = 0xFE;    //a
            LATA = 0x10;
            L_wait(10);
            if((PORTBbits.RB0 == 0) && (i == 0)) i = 1;

            LATC = 0xDF;   //f
            LATA = 0x08;
            L_wait(10);
            LATC = 0xFD;    //b
            LATA = 0x10;
            L_wait(10);
            if((PORTBbits.RB0 == 0) && (i == 0)) i = 2;

            LATC = 0xFB;   //c
            LATA = 0x00;
            L_wait(10);
            LATC = 0xEF;    //e
            LATA = 0x18;
            L_wait(10);
            if((PORTBbits.RB0 == 0) && (i == 0)) i = 3;

            LATC = 0xF7;   //d
            LATA = 0x00;
            L_wait(10);
            LATC = 0xF7;    //d
            LATA = 0x18;
            L_wait(10);
            if((PORTBbits.RB0 == 0) && (i == 0)) i = 4;

            LATC = 0xEF;   //e
            LATA = 0x00;
            L_wait(10);
            LATC = 0xFB;    //c
            LATA = 0x18;
            L_wait(10);
            if((PORTBbits.RB0 == 0) && (i == 0)) i = 5;

            LATC = 0xDF;   //f
            LATA = 0x00;
            L_wait(10);
            LATC = 0xFD;    //b
            LATA = 0x18;
            L_wait(10);
            if((PORTBbits.RB0 == 0) && (i == 0)) i = 6;
        }



        LATA = 0x38;   // LEDを消す
        while(PORTBbits.RB0 == 0);   // SW1を離すまで待つ
        L_wait(10);   // チャタリング防止
        if(i == 1) LATC = 0xBF;
        else if(i == 2) LATC = 0xF6;
        else if(i == 3) LATC = 0xB6;
        else if(i == 4) LATC = 0xD2;
        else if(i == 5) LATC = 0x92;
        else if(i == 6) LATC = 0xC0;
        LATA = 0x20;
        while(PORTBbits.RB0 == 1);   // SW1を押すまで待つ
        L_wait(10);   // チャタリング防止
        while(PORTBbits.RB0 == 0);   // SW1を離すまで待つ
        L_wait(10);   // チャタリング防止
        i = 0;
    }

}



//////////////////////////////////////////////////////////////////////////////////////
//   #12 Sound Switch
//////////////////////////////////////////////////////////////////////////////////////
//
// 大きな音をマイクで拾うと RA2 が H になる
// 音が入ると、8bitLEDの光る位置が１つずつ左シフトする（デフォルト、SW1）
// SW2を押すと、音が入る毎にフルカラーLEDの色が変わる
// SW3を押すと、音が入った回数が７セグメントLED（右側）に表示される
// マイクの近くで手を叩く、あるいはマイクをトントンとタッピングするとよい
//
void SoundSwitch(void){

    unsigned char s, n, mode, i;
    unsigned char p10, p1;

    TRISB = 0x8F;  // RB5を出力に変更
    PIE4bits.TMR2IE = 1;  // TMR2割り込みを使用
    INTCONbits.PEIE = 1;
    INTCONbits.GIE  = 1;

    s = 0;
    n = 0;
    mode = 0;
    LATC = 0xFF;
    LATA = 0x30; // select 8-LEDs

    p1 = 0;
    p10 = 0;
    i = 0;


    while(1){

        if(mode == 0){  // 8bitLEDの場合
            if(PORTAbits.RA2 == 1){  // 音が鳴った場合
                if(n == 0) n = 1;  // 1bit左シフト
                else n <<= 1;
                while(PORTAbits.RA2 == 1);  // 信号レベルが下がるまで待つ
                LATC = ~n;
                t2_icnt = 0;
                while(t2_icnt < 250);  // チャタリング防止
            }
        }
        else if(mode == 1){
            if(PORTAbits.RA2 == 1){  // 音が鳴った場合
                n++;  // カウントアップ
                while(PORTAbits.RA2 == 1);  // 信号レベルが下がるまで待つ
                LATC = ~n;
                t2_icnt = 0;
                while(t2_icnt < 250);  // チャタリング防止
            }
        }
        else if(mode == 3){
            if(PORTAbits.RA2 == 1){  // 音が鳴った場合
                p1++;  // 回数をカウントアップ
                if(p1 == 10){
                    p1 = 0;
                    p10++;
                }
                while(PORTAbits.RA2 == 1);  // 信号レベルが下がるまで待つ
                t2_icnt = 0;
                while(t2_icnt < 250);  // チャタリング防止
            }
            if(i < 128){  // ７セグメントLED（右側）をダイナミック点灯
                LATC = 0xFF;
                LATA = 0x10;
                LATC = segdata[p10];
            }
            else{
                LATC = 0xFF;
                LATA = 0x18;
                LATC = segdata[p1];
            }
            i++;
        }
        //
        if(PORTBbits.RB0 == 0){  // SW1が押された場合
            mode = 0;
            n = 0;
            LATC = 0xFF;
            LATA = 0x30;   // select 8-LEDs
            while(PORTBbits.RB0 == 0);
        }
        if(PORTBbits.RB1 == 0){  // SW2が押された場合
            mode = 1;
            n = 0;
            LATC = 0xFF;
            LATA = 0x28;   // select full color LED
            while(PORTBbits.RB1 == 0);
        }
        if(PORTBbits.RB2 == 0){  // SW3が押された場合
            mode = 3;
            p1 = 0;
            p10 = 0;
            while(PORTBbits.RB0 == 0);
        }
    }


}




//////////////////////////////////////////////////////////////////////////////////////
//   #13 A/D converter
//////////////////////////////////////////////////////////////////////////////////////
//
// 可変抵抗（VR）の電圧（RA0）をA/D変換し、0 -- 1023 の数値を７セグメントLEDで表示する
// 同時に、電圧レベルを8bitLEDでバーグラフ表示する
//
void ADconverter(void){
    
    unsigned int ad_result;
    unsigned int m;


    TRISB = 0x8F;   // RB5を出力に変更
    ADPCH = 0;        // select ANA0


    while(1){

        ADCON0bits.ADGO = 1;    // A/D 変換を開始する //
        while(ADCON0bits.ADGO);
        ad_result = (ADRESH << 8);
        ad_result += ADRESL;

        LATC = 0xFF; // A/D変換結果に応じて8bitLEDでバーグラフ表示
        if(ad_result < 64) LATC = 0xFF;
        else if(ad_result < 192) LATC = 0xFE;
        else if(ad_result < 320) LATC = 0xFC;
        else if(ad_result < 448) LATC = 0xF8;
        else if(ad_result < 576) LATC = 0xF0;
        else if(ad_result < 704) LATC = 0xE0;
        else if(ad_result < 832) LATC = 0xC0;
        else if(ad_result < 960) LATC = 0x80;
        else LATC = 0x00;
        LATA = 0x30;

        bin16_to_bcd(ad_result);    // A/D変換結果を１０進数に直す

        // ７セグメントLED（４桁）をダイナミック点灯
        LATC = 0xFF;
        LATA = 0x00;
        LATC = segdata[bcd[3]];    // 1000の位

        for(m=0; m<500; m++);

        LATC = 0xFF;
        LATA = 0x08;
        LATC = segdata[bcd[2]];    // 100の位

        for(m=0; m<500; m++);

        LATC = 0xFF;
        LATA = 0x10;
        LATC = segdata[bcd[1]];    // 10の位

        for(m=0; m<500; m++);

        LATC = 0xFF;
        LATA = 0x18;
        LATC = segdata[bcd[0]];    // 1の位

        for(m=0; m<500; m++);


    }

}




//////////////////////////////////////////////////////////////////////////////////////
//   #14 RC servo
//////////////////////////////////////////////////////////////////////////////////////
//
// 可変抵抗（VR）の電圧（RA0）をA/D変換し、電圧レベルに応じて RC0 のパルス幅を変化させる
// パルスの繰り返し周期は 15ms
// 可変抵抗のツマミを回すのに応じてRCサーボのサーボホーンが回転する
//
void RCservo(void){

    unsigned int ad;
    unsigned char a;

    TRISB = 0x8F;   // RB5を出力に変更
    PIE4bits.TMR2IE = 1;   // use TMR2 interrupt
    INTCONbits.PEIE = 1;
    INTCONbits.GIE = 1;

    t2_icnt = 0;
    LATC = 0xFE;

    while(1){

        ADPCH = 0;        // select ANA0 (VR)
        ADCON0bits.ADGO = 1;
        while(ADCON0bits.ADGO);
        ad = ADRESH << 8;
        ad += ADRESL;
        a = ad >> 2;
        if(a == 0) a = 1;   // 最小値を１に制限

        while(t2_icnt < 75);   // period = 15ms
        t2_icnt = 0;
        LATC = 0xFF;   // make RC0 1
        wait(200);  // パルス幅の最小値
        wait(a);    // A/D変換結果に応じてパルス幅を変える
        wait(a);
        LATC = 0xFE;   // make RC0 0

    }
    
}


//////////////////////////////////////////////////////////////////////////////////////
//   #15 Timer
//////////////////////////////////////////////////////////////////////////////////////
//
// SW4 を押しながら SW3 で時間の値の設定（１ずつ増加するだけ）
// SW4 を押しながら SW2 で10分の位の値の設定（１ずつ増加するだけ）
// SW4 を押しながら SW1 で1分の位の値の設定（１ずつ増加するだけ）
// SW1 でスタート／ポーズ
// カウント終了でアラーム、電源切るまで鳴り続ける
// 一旦カウントをスタートさせると、途中でポーズはできるが設定時間の変更はできない
// １秒毎にサイコロLEDが点滅、5秒毎に点滅箇所が移動、30秒経過後は中央の赤LEDも一緒に点滅
//
void Timer(void){
    
    unsigned char hour10, hour1, min10, min1, s, kf, status;
    unsigned char i;
    
    TRISB = 0x8F;   // RB5を出力に変更

    PIE4bits.TMR2IE = 1;  // use TMR2 interrupt
    INTCONbits.PEIE = 1;
    INTCONbits.GIE  = 1;

    hour10 = 0;
    hour1 = 0;
    min10 = 0;
    min1 = 0;
    s = 0;
    kf = 2;
    sec_flag = 0;
    t2_icnt = 0;
    status = 0; //0: stand by, 1: running, 2: end, 3: pause
    
    while(1){
        for(i = 0; i < 255; i++){  // ７セグメントLEDとサイコロLEDをダイナミック点灯
            LATC = 0xFF;
            if(i < 51){
                if(hour10 > 0) LATC = segdata[hour10];
                LATA = 0x00;
            }
            else if(i < 102){
                LATC = 0x7F & segdata[hour1];  // 「：」は常時点灯
                LATA = 0x08;
            }
            else if(i < 153){
                LATC = segdata[min10];
                LATA = 0x10;
            }
            else if(i < 204){
                LATC = segdata[min1];
                LATA = 0x18;
            }
            else{
                if(status == 1){
                    if(t2_icnt < 2500){ // サイコロLEDの点灯は0.5秒間のみ
                        if(s <= 5) LATC = 0xFE;
                        else if(s <= 10) LATC = 0xFD;
                        else if(s <= 15) LATC = 0xFB;
                        else if(s <= 20) LATC = 0xF7;
                        else if(s <= 25) LATC = 0xEF;
                        else if(s <= 30) LATC = 0xDF;
                        else if(s <= 35) LATC = 0xBE;   // 30秒経過後は中央の赤LEDも一緒に点滅
                        else if(s <= 40) LATC = 0xBD;
                        else if(s <= 45) LATC = 0xBB;
                        else if(s <= 50) LATC = 0xB7;
                        else if(s <= 55) LATC = 0xAF;
                        else LATC = 0x9F;
                    }
                }
                LATA = 0x20;
            }
            if(status == 1){
                if(sec_flag == 1){  // １秒経過時の処理
                    sec_flag = 0;
                    s++;    // 秒の値をカウントアップ
                    if(s == 60){    // １分経過時の処理
                        s = 0;
                        if(min1 > 0){
                            min1--; // 分の値をカウントダウン
                            if(min1 == 0){
                                if(min10 == 0){
                                    if(hour1 == 0){
                                        if(hour10 == 0) status = 2; // 全部ゼロになったら終了
                                    }
                                }
                            }
                        }
                        else{ // case min1 = 0
                            if(min10 > 0){
                                min10--;
                                min1 = 9;
                            }
                            else{   // case min10 = 0
                                if(hour1 > 0){
                                    hour1--;
                                    min10 = 5;
                                    min1 = 9;
                                }
                                else{   // case hour1 = 0
                                    if(hour10 > 0){
                                        hour10--;
                                        hour1 = 9;
                                        min10 = 5;
                                        min1 = 9;
                                    }
                                    else{
                                        status = 2; // end of time count
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        //
        if(status == 0){    // 時間のセット
            if((PORTB & 0x0F) == 0x07) kf = 0;
            if((PORTB & 0x0F) > 0x07) kf = 2;    // make change inactive
            if(kf == 0){    // SW4が押されている場合
                if(PORTBbits.RB2 == 0){ // SW3も押されている場合
                    kf = 1;
                    hour1++;
                    if(hour1 == 10){
                        hour1 = 0;
                        hour10++;
                        if(hour10 == 10) hour10 = 0;
                    }
                }
                if(PORTBbits.RB1 == 0){ // SW2も押されている場合
                    kf = 1;
                    min10++;
                    if(min10 == 6) min10 = 0;
                }
                if(PORTBbits.RB0 == 0){ // SW1も押されている場合
                    kf = 1;
                    min1++;
                    if(min1 == 10) min1 = 0;
                }
            }
            if(kf == 2){    // SW4が離されている場合
                if(PORTBbits.RB0 == 0){ // SW1が押された場合
                    if((min1 > 0) || (min10 > 0) || (hour1 > 0) || (hour10 > 0)){
                        status = 1;
                        kf = 3;
                        t2_icnt = 0;
                        s = 0;
                    }
                }
            }
        }
        //
        if(status == 1){    // カウント中にSW1が押されたらポーズ
            if((PORTB & 0x0F) == 0x0F) kf = 0;
            if(kf == 0){
                if(PORTBbits.RB0 == 0){
                    status = 3;
                    kf = 3;
                }
            }
        }
        //
        if(status == 2) ararm();    // カウント終了したらアラームを鳴らす
        //
        if(status == 3){    // ポーズ中にSW1が押されたら再スタート
            if((PORTB & 0x0F) == 0x0F) kf = 0;
            if(kf == 0){
                if(PORTBbits.RB0 == 0){
                    status = 1;
                    kf = 3;
                }
            }            
        }
    }
      
}



////////////////////////////////////////////////////////////////////////////////////////

void wait(unsigned char n){
    unsigned char i;
    for(i=0; i< n; i++);
#ifndef XTAL20MHz
    if(n>1){
        n >>= 1;
        for(i=0; i< n; i++);
    }
#endif
}

void L_wait(unsigned char n){
    unsigned char i;
    for(i=0; i<n; i++){
        wait(255);
    }
}

void ararm(void){

    unsigned char a, b, c, d;

    TRISB = 0x8B;   // make RB2 output

    d = 0x55;
    LATA = 0x30;   // select 8-LEDs

    while(1){

        for(b = 0; b < 10; b++){
            LATC = d;
            d = ~d;
            for(c = 0; c < 100; c++){
                LATB = 0x18;
                wait(64);
                LATB = 0x08;
                wait(64);
            }
            for(c = 0; c < 100; c++){
                LATB = 0x18;
                wait(48);
                LATB = 0x08;
                wait(48);
            }
        }
        L_wait(100) ;

    }

}

unsigned char random(void){
    unsigned char i, a, b;

    ADPCH = 1;        // select ANA1 (CdS)
    ADCON0bits.ADGO = 1;
    while(ADCON0bits.ADGO);
    a = ADRESL;            // A/D 変換結果の上位 8bit のみに注目 //
    b = a;
    for(i=0; i<7; i++){ // MSB <--> LSB 入れ替え
        a >>= 1;
        b <<= 1;
        if((a & 0x01) == 0x01) b |= 0x01;
    }
    b += TMR0;  // TMR0の値も加えてランダム性改善

    return b;

}

void bin16_to_bcd(unsigned int bin16)
{
    unsigned char i, j, s;

    for(i=0; i<5; i++) bcd[i] = 0;

    for(j=0; j<16; j++){
        s = 1;
        if(bin16 < 0x8000) s = 0;
        bin16 += bin16;		//shift left

        for(i=0; i<5; i++){
            bcd[i] += bcd[i]; 	 //shift left
        }
        bcd[0] += s;
        for(i=0; i<4; i++){
            if(bcd[i] > 15){
                bcd[i] &= 0x0F;
                bcd[i+1]++;
            }
        }
        if(j < 15){
            for(i=0; i<5; i++){
                if(((bcd[i]+3) & 0x08) !=0) bcd[i] += 3;
            }
        }

    }

}






